C  ****************************************************************************
C  *                                                                          *
C  *                        *****     MAPLIB.F     *****                      *
C  *                                                                          *
C  *    THIS LIBRARY CONTAINS SUBROUTINES USED TO MANIPULATE "MAP" ARRAY      *
C  *     DATA.  THE OBJECT CODE FOR ALL THE ROUTINES ARE KEPT IN THE          *
C  *     LIBRARY TSBLIB.OLB                                                   *
C  *                                                                          *
C  *    LIST OF SUBROUTINES:                                                  *
C  *                                                                          *
C  *      MAP_ADD, MAP_CLEAR,  MAP_DIF,                                       *
C  *      MAP_HEADER, MAP_FFT, MAP_FFT_FILL, MAP_FIX_*,                       *
C  *      MAP_GET_ANGLE, MAP_IMG, MAP_MULTIPLY, MAP_NOISE, MAP_PEAK,          *
C  *      MAP_PEAKSEARCH, MAP_POLAR, MAP_POLAR_GRID, MAP_PRJ, MAP_PRJ_XZ,     *
C  *      MAP_PRJ_AXIS, MAP_PRJ_ALL, MAP_ROT, MAPI2_ROT, MAP_ROTX,            *
C  *      MAP_ROTX_SLOW, MAP_ROTY, MAP_ROTY_SLOW, MAP_SCALE, MAP_SHIFT,       *
C  *      MAP_STATS, MAP_SYM, MAP_SYM_CAVG, MAP_SYM_RAVG, MAP_SYM_GRID,       *
C  *      MAP_TRP, MAP_VOL, MAP_XFLIP, MAP_YFLIP, MAP_1D                      *
C  *                                                                          *
C  *    MAY 1988  PURDUE UNIVERSITY VAX 8550, T. BAKER                        *
C  ****************************************************************************
C
C
C
C
C

C  ******************************************************************
C  *                                                                *
C  *                *****  SUBROUTINE MAP_ADD  *****                *
C  *                                                                *
C  *    COMPUTE MAP1 + MAP2 AND STORE ANSWER IN MAP2.               *
C  *                                                                *
C  *    SEPTEMBER 1986  PURDUE UNIVERSITY VAX 11/750, L. MATTERN.   *
C  *    AUGUST 1990  PURDUE UNIVERSITY VAX 8550, T. BAKER.          *
C  *    LAST MODIFIED 17-DEC-1997 BY T. BAKER (RID OF EM.CMM).      *
C  ******************************************************************
C
      SUBROUTINE MAP_ADD (MAP1, MAP2, NCOL, NROW, MAP_MIN, MAP_MAX, 
     1 MAP_AVG, MAP_STD, EM_WRT)
C
      IMPLICIT NONE
C
      INTEGER*4 NCOL, NROW, I, ARRAY_SIZE, EM_WRT
      REAL*4    MAP1(1), MAP2(1), MAP_MIN, MAP_MAX, MAP_AVG, MAP_STD
C -----------------------------------------------------------------------------
C
      ARRAY_SIZE = NCOL*NROW
      IF(EM_WRT.EQ.1) THEN
        PRINT *, '**********************************************'
        PRINT *, '* Compute MAP1+MAP2 and store answer in MAP2 *'
        PRINT *, '**********************************************'
      ENDIF
C
      DO I = 1,ARRAY_SIZE
        MAP2(I) = MAP1(I) + MAP2(I)
      ENDDO
C
      CALL MAP_STATS (1, MAP2, NCOL, NROW, 1, MAP_MIN, MAP_MAX, 
     1 MAP_AVG, MAP_STD, EM_WRT)
C
      RETURN
      END

C  ********************************************************************
C  *                                                                  *
C  *             *****     SUBROUTINE MAP_CLEAR     *****             *
C  *                                                                  *
C  *    ZEROES THE NCOL BY NROW BY NSEC "MAP" ARRAY.                  *
C  *                                                                  *
C  *    APRIL 1984  PURDUE UNIVERSITY VAX 11/750, T. BAKER.           *
C  *    MAY 1989  PURDUE UNIVERSITY VAX 8550, T. BAKER.               *
C  *    LAST MODIFIED 12-MAY-89 BY T. BAKER                           *
C  ********************************************************************
C
      SUBROUTINE MAP_CLEAR (MAP, NCOL, NROW, NSEC)
C
      IMPLICIT NONE
C
      INTEGER*4 I, NCOL, NROW, NSEC, ARRAY_SIZE
      REAL*4    MAP(1), ZERO
C  ----------------------------------------------------------------------------
C
      ARRAY_SIZE = NCOL*NROW*NSEC
      ZERO       = 0.0
C
      DO I = 1,ARRAY_SIZE
        MAP(I) = ZERO
      ENDDO
C
      RETURN
      END



C  ****************************************************************
C  *                                                              *
C  *            *****     SUBROUTINE MAP_DIF     *****            *
C  *                                                              *
C  *    COMPUTE MAP1-MAP2 AND STORE ANSWER IN MAP2                *
C  *                                                              *
C  *    JUNE 1986  PURDUE UNIVERSITY VAX 11/750, T. BAKER         *
C  *    LAST MODIFIED 17-DEC-1997  BY T. BAKER (RID OF EM.CMM).   *
C  ****************************************************************
C
      SUBROUTINE MAP_DIF (MAP1, MAP2, NCOL, NROW, MAP_MIN, MAP_MAX, 
     1 MAP_AVG, MAP_STD, EM_WRT)
C
      IMPLICIT NONE
C
      INTEGER*4 I, NCOL, NROW, ARRAY_SIZE, EM_WRT
      REAL*4    MAP1(1), MAP2(1), MAP_MIN, MAP_MAX, MAP_AVG, MAP_STD
C  ----------------------------------------------------------------------------
C
      ARRAY_SIZE = NCOL*NROW
      IF(EM_WRT.EQ.1) WRITE(*,1000)
C
      DO I = 1,ARRAY_SIZE
        MAP2(I) = MAP1(I) - MAP2(I)
      ENDDO
C
      CALL MAP_STATS (1, MAP2, NCOL, NROW, 1, MAP_MIN, MAP_MAX, 
     1 MAP_AVG, MAP_STD, EM_WRT)
C
      RETURN
C
 1000      FORMAT(' Compute MAP1-MAP2 and store answer in MAP2')
      END

C  ******************************************************************
C  *                                                                *
C  *            *****     SUBROUTINE MAP_HEADER     *****           *
C  *                                                                *
C  *    READS MAP FILE HEADER DATA ONLY                             *
C  *                                                                *
C  *    APRIL 1986  PURDUE UNIVERSITY VAX 11/750, T. BAKER          *
C  *    LAST MODIFIED 15-DEC-1997 BY T. BAKER (RID OF EM.CMM).      *
C  ******************************************************************
C
      SUBROUTINE MAP_HEADER (LUN, NCOL, NROW, NSEC, FILE_ID,
     1 MAP_TITLE, MAP_MIN, MAP_MAX, AOVERB, MAP_ABANG, A_CELL, 
     2 B_CELL, C_CELL, ALP, BET, GAM, DELA, DELB, DELC, T_MATRIX)
C
      IMPLICIT NONE
C
      INTEGER*4 NCOL, NROW, NSEC, LUN, FILE_ID, MAP_TITLE(18)
      INTEGER*4 T_MATRIX(2,3)
      REAL*4    MAP_MIN, MAP_MAX, AOVERB, MAP_ABANG
      REAL*4    A_CELL, B_CELL, C_CELL, ALP, BET, GAM, DELA, DELB, DELC
C  ----------------------------------------------------------------------------
C
      READ(LUN) FILE_ID
      READ(LUN) MAP_TITLE
      READ(LUN) NCOL, NROW, NSEC, MAP_MIN, MAP_MAX, AOVERB, MAP_ABANG
      READ(LUN) A_CELL, B_CELL, C_CELL, ALP, BET, GAM
      READ(LUN) DELA, DELB, DELC, T_MATRIX
      RETURN
C
      END

C  ********************************************************************
C  *                                                                  *
C  *              *****     SUBROUTINE MAP_FFT     *****              *
C  *                                                                  *
C  *    LOAD NCOL X NROW "MAP" INTO "FFT" ARRAY AND COMPUTE FFT.      *
C  *                                                                  *
C  *    JUNE 1991  PURDUE UNIVERSITY VAX 8550, T. BAKER.              *
C  *    MODIFIED 17-DEC-1997 BY T. BAKER (Get rid of EM.CMM).         *
C  *    LAST MODIFIED 30-NOV-1999 BY TSB (Get rid of FFT_IEND).       *
C  ********************************************************************
C
      SUBROUTINE MAP_FFT (MAP, NCOL, NROW, NSEC, SEC, FFT, IDIM1, 
     1 IDIM2, FFT_ORIGX, FFT_ORIGY, FFT_FZERO, FFT_FULL)
C
      IMPLICIT NONE
C
      INTEGER*4 NCOL, NROW, NSEC, SEC, IDIM1, IDIM2, EM_WRT
      REAL*4    MAP(NCOL,NROW,NSEC)
      REAL*4    FFT(2*IDIM1,IDIM2), FFT_ORIGX, FFT_ORIGY, FFT_FZERO
      LOGICAL   FFT_FULL
C  -----------------------------------------------------------------------------
C
      EM_WRT = 0
      CALL MAP_FFT_FILL (MAP, NCOL, NROW, NSEC, SEC, FFT, IDIM1, IDIM2, 
     1 FFT_FZERO, FFT_ORIGX, FFT_ORIGY, FFT_FULL, EM_WRT)
      CALL FFT_2D (FFT, IDIM1, IDIM2)
C
      RETURN
      END

C  **********************************************************************
C  *                                                                    *
C  *             *****     SUBROUTINE MAP_FFT_FILL     *****            *
C  *                                                                    *
C  *    LOAD NCOL X NROW "MAP" SECTION INTO "FFT" ARRAY TO COMPUTE FFT  *
C  *                                                                    *
C  *    BEWARE: IT IS ***IMPORTANT*** THAT THE DATA BE PROPERLY FLOATED *
C  *            BEFORE THE DATA ARE FOURIER TRANSFORMED.                *
C  *                                                                    *
C  *    MAY 1989  PURDUE UNIVERSITY VAX 8550, T. BAKER                  *
C  *    MODIFIED 17-DEC-1997 BY T. BAKER (Get rid of EM.CMM).           *
C  *    LAST MODIFIED 30-NOV-1999 BY TSB (Get rid of FFT_IEND).         *
C  **********************************************************************
C
      SUBROUTINE MAP_FFT_FILL (MAP, NCOL, NROW, NSEC, SEC, FFT, IDIM1, 
     1 IDIM2, FFT_FZERO, FFT_ORIGX, FFT_ORIGY, FFT_FULL, EM_WRT)
C
      IMPLICIT NONE
C
      INTEGER*4 NCOL, NROW, NSEC, SEC, IDIM1, IDIM2, I, J, EM_WRT
      REAL*4    MAP(NCOL,NROW,NSEC), FFT(2*IDIM1,IDIM2), TMP
      REAL*4    FFT_FZERO, FFT_ORIGX, FFT_ORIGY
      LOGICAL   FFT_FULL
C  -----------------------------------------------------------------------------
C
      IF(FFT_ORIGX.EQ.0.0 .AND. FFT_ORIGY.EQ.0.0) THEN
        FFT_ORIGX = (NCOL-1)/2.0
        FFT_ORIGY = (NROW-1)/2.0
      ENDIF
      FFT_FZERO = 0.0
C
      FFT_FULL = .TRUE.
      CALL FFT_CLEAR (FFT, IDIM1, IDIM2, FFT_FULL)         ! ZERO THE FFT ARRAY
C
      DO J = 1,NROW
        DO I = 1,NCOL
          TMP            = MAP(I,J,SEC)
          FFT((2*I)-1,J) = TMP
          FFT_FZERO      = FFT_FZERO + TMP
        ENDDO
      ENDDO
C
      IF(EM_WRT.EQ.1) WRITE(*,1000) FFT_FZERO
 1000      FORMAT(' F-ZERO OF MAP SECTION =',E12.3,'            ')
C
      RETURN
      END

C  ************************************************************************
C  *                                                                      *
C  *                  *****  SUBROUTINE MAP_FIX_A  *****                  *
C  *                                                                      *
C  *    A  ADD a constant to MAP values.                                  *
C  *                                                                      *
C  *    FEBRUARY 1992  PURDUE UNIVERSITY VAX 8550 T. BAKER.               *
C  *    MODIFIED 16-DEC-1997 BY T. BAKER (Rid of EM.CMM).                 *
C  *    LAST MODIFIED 27-FEB-1998 BY T. BAKER (Make UNIX compatible).     *
C  ************************************************************************
C
      SUBROUTINE MAP_FIX_A (LUN, MAP, NCOL, NROW, NSEC, TMP, 
     1 MAP_INCORE)
C
      IMPLICIT NONE
C
      INTEGER*4 NCOL, NROW, NSEC, LUN, I, J, K, N, NUMCHARI4
      INTEGER*4 COL1, COL2, ROW1, ROW2, SEC1, SEC2, ITMP
      REAL*4    MAP(NCOL,NROW,NSEC), TMP(NCOL), PERCENT, CONSTANT
      REAL*4    DIV
      LOGICAL   NOTIFY, MAP_INCORE
C ------------------------------------------------------------------------------
C
      IF(.NOT.MAP_INCORE) THEN
        PRINT *, '*************************************************'
        PRINT *, '* MAP not in core memory - code can not be used *'
        PRINT *, '*************************************************'
        RETURN
      ENDIF
Ca      CALL VT_CLS
      PERCENT = 0.0
      COL1    = 1
      COL2    = NCOL
      ROW1    = 1
      ROW2    = NROW
      SEC1    = 1
      SEC2    = NSEC
C
 100  WRITE(*,'(A)') '$CONSTANT TO ADD (F): '
      READ (*,*,ERR=100) CONSTANT
      IF(CONSTANT.EQ.0.0) RETURN
C
 200  IF(NCOL.GT.1) THEN
         PRINT *, '*********************************************'
         PRINT *, '* Enter the starting and ending col numbers *'
         PRINT *, '*********************************************'
         N = NUMCHARI4 (NCOL)
 300     WRITE(*,'(A,I6,A)') '$COL1,COL2 (<CR>=1,', NCOL, ': '
         READ (*,*,ERR=300) COL1, COL2
         IF(COL1.EQ.0 .AND. COL2.EQ.0) THEN
           COL1 = 1
           COL2 = NCOL
         ENDIF
         IF(COL1.GT.COL2) THEN
           ITMP = COL1
           COL1 = COL2
           COL2 = ITMP
         ENDIF
         IF(COL1.LE.0 .OR. COL2.GT.NCOL) THEN
           CALL BELL (2)
           PRINT *, '***********************************'
           PRINT *, '* Number of columns out of bounds *'
           PRINT *, '***********************************'
           GO TO 300
         ENDIF
       ELSE
         COL1 = 1
         COL2 = 1
      ENDIF
C
      IF(NROW.GT.1) THEN
         PRINT *, '*********************************************'
         PRINT *, '* Enter the starting and ending row numbers *'
         PRINT *, '*********************************************'
         N = NUMCHARI4 (NROW)
 400     WRITE(*,'(A,I7,A)') '$ROW1,ROW2 (<CR>=1,', NROW, ': '
         READ (*,*,ERR=400) ROW1, ROW2
         IF(ROW1.EQ.0 .AND. ROW2.EQ.0) THEN
           ROW1 = 1
           ROW2 = NROW
         ENDIF
         IF(ROW1.GT.ROW2) THEN
           ITMP = ROW1
           ROW1 = ROW2
           ROW2 = ITMP
         ENDIF
         IF(ROW1.LE.0 .OR. ROW2.GT.NROW) THEN
           CALL BELL (2)
           PRINT *, '********************************'
           PRINT *, '* Number of rows out of bounds *'
           PRINT *, '********************************'
           GO TO 400
         ENDIF
       ELSE
         ROW1 = 1
         ROW2 = 1
      ENDIF
C
      IF(NSEC.GT.1) THEN
         PRINT *, '*************************************************'
         PRINT *, '* Enter the starting and ending section numbers *'
         PRINT *, '*************************************************'
         N = NUMCHARI4 (NSEC)
 500     WRITE(*,'(A,I7,A)') '$SEC1,SEC2 (<CR>=1,',NSEC, ': '
         READ (*,*,ERR=500) SEC1, SEC2
         IF(SEC1.EQ.0 .AND. SEC2.EQ.0) THEN
           SEC1 = 1
           SEC2 = NSEC
         ENDIF
         IF(SEC1.GT.SEC2) THEN
           ITMP = SEC1
           SEC1 = SEC2
           SEC2 = ITMP
         ENDIF
         IF(SEC1.LE.0 .OR. SEC2.GT.NSEC) THEN
           CALL BELL (2)
           PRINT *, '************************************'
           PRINT *, '* Number of sections out of bounds *'
           PRINT *, '************************************'
           GO TO 500
         ENDIF
       ELSE
         SEC1 = 1
         SEC2 = 1
      ENDIF
      DIV = FLOAT(SEC2-SEC1+1)
      IF(SEC2.GT.SEC1) THEN
         NOTIFY = .TRUE.
         WRITE(*,'(F6.1,''% DONE'')') PERCENT
       ELSE
         NOTIFY = .FALSE.
      ENDIF
C
      DO K = SEC1,SEC2
        DO J = ROW1,ROW2
          DO I = COL1,COL2
            MAP(I,J,K) = MAP(I,J,K) + CONSTANT
          ENDDO
        ENDDO
        IF(NOTIFY) THEN
          PERCENT = 100.0*FLOAT(K-SEC1+1)/DIV
          WRITE(*,'(''+'',F6.1)') PERCENT
        ENDIF
      ENDDO
C
      RETURN
      END

C  **********************************************************************
C  *                                                                    *
C  *                 *****  SUBROUTINE MAP_FIX_F  *****                 *
C  *                                                                    *
C  *    F  Apply Fourier filter to MAP sections.                        *
C  *                                                                    *
C  *    JANUARY 1992  PURDUE UNIVERSITY VAX 8550 T. BAKER.              *
C  *    MODIFIED 15-DEC-1997 BY T. BAKER (RID OF EM.CMM).               *
C  *    LAST MODIFIED 25-FEB-1998 BY T. BAKER (Add UNIX compatibility). *
C  *    LAST MODIFIED 30-NOV-1999 BY TSB (Get rid of FFT_IEND).         *
C  **********************************************************************
C
      SUBROUTINE MAP_FIX_F (MAP, NCOL, NROW, NSEC, FFT_ORIGX,
     1 FFT_ORIGY, FFT_FZERO, FFT_FULL)
C
      IMPLICIT NONE
C
      INTEGER*4 NCOL, NROW, NSEC, IDIM1, IDIM2, I
      REAL*4    MAP(NCOL,NROW,NSEC), RESMIN, RESMAX, CUTMIN, CUTMAX
      REAL*4    FFT_ORIGX, FFT_ORIGY, FFT_FZERO
      real*4, allocatable :: fft1(:)
      integer   alloc_stat
      LOGICAL   FFT_FULL
C       -----------------------------------------------------------------------
C
      PRINT *,'*****************************************'
      PRINT *,'* WARNING: THIS ROUTINE ONLY WORKS WITH *'
      PRINT *,'*            ORTHOGONAL DATA            *'
      PRINT *,'*****************************************'
      PRINT *
C
      CALL FFT_SETDIM_DEF_SAME (NCOL, NROW, IDIM1, IDIM2, 1)
 50   WRITE(*,'(A)') '$IS THIS SIZE OK<CR> OR DOUBLE IT(1): '
      READ (*,*,ERR=50) I
      IF(I.EQ.1) THEN
         IDIM1 = 2*IDIM1
         IDIM2 = IDIM1
      ENDIF
      FFT_FULL = .TRUE.

      allocate(fft1(2*idim1*idim2), stat=alloc_stat)
      if(alloc_stat > 0) then
         write(*,*) 'Unable to allocate FFT1 in MAP_FIX_F'
         call tsbend
      endif

      PRINT *, '****************************************'
      PRINT *, '* Option for Fourier Filtering         *'
      PRINT *, '*                                      *'
      PRINT *, '* RESMIN,RESMAX,CUTMIN,CUTMAX          *'
      PRINT *, '* RESMIN is the HI-PASS cutoff         *'
      PRINT *, '* RESMAX is the LO-PASS cutoff         *'
      PRINT *, '* CUTMIN and CUTMAX apply soft cutoffs *'
      PRINT *, '* <CR>= no filter                      *'
      PRINT *, '****************************************'
 100  WRITE(*,'(A,F6.1,A)') '$RESMIN,RESMAX,CUTMIN,CUTMAX (4F) (Max=',
     1 FLOAT(IDIM1/2), '): '
      READ (*,*,ERR=100) RESMIN, RESMAX, CUTMIN, CUTMAX
      IF(RESMIN.LT.0.0 .OR. RESMAX.LT.0.0) GO TO 100
      IF(CUTMIN.LE.0.0) CUTMIN = 0.0
      IF(CUTMAX.LE.0.0) CUTMAX = 0.0
      IF((RESMIN-CUTMIN).LT.0.0) CUTMIN = RESMIN 
      IF(RESMIN.EQ.0.0 .AND. RESMAX.EQ.0.0) GO TO 900
C
      DO I = 1,NSEC
         CALL MAP_FFT_FILL (MAP, NCOL, NROW, NSEC, I, FFT1, IDIM1, 
     1        IDIM2, FFT_FZERO, FFT_ORIGX, FFT_ORIGY, FFT_FULL, 0)
         CALL FFT_2D (FFT1, IDIM1, IDIM2)
         IF(RESMAX.GT.0.0) THEN
            CALL FFT_LOP (FFT1, IDIM1, IDIM2, RESMAX, CUTMAX)
         ENDIF
         IF(RESMIN.GT.0.0) THEN
            CALL FFT_HIP (FFT1, IDIM1, IDIM2, RESMIN, CUTMIN)
         ENDIF
         CALL FFT_2D_BACK (FFT1, IDIM1, IDIM2)
         CALL FFT_MAP_FILL (FFT1,IDIM1,IDIM2,MAP,NCOL,NROW,NSEC,I)
      ENDDO

      deallocate(fft1)

 900  RETURN
      END

C  ************************************************************************
C  *                                                                      *
C  *                  *****  SUBROUTINE MAP_FIX_M  *****                  *
C  *                                                                      *
C  *    M  Multiply section(s) by a constant.                             *
C  *                                                                      *
C  *    AUGUST 1990  PURDUE UNIVERSITY VAX 8550 T. BAKER.                 *
C  *    MODIFIED 16-DEC-1997 BY T. BAKER (RID OF EM.CMM).                 *
C  *    LAST MODIFIED 27-FEB-1998 BY T. BAKER (Make UNIX compatible).     *
C  ************************************************************************
C
      SUBROUTINE MAP_FIX_M (LUN, MAP, NCOL, NROW, NSEC, WEIGHTS, TMP,
     1 MAP_MIN, MAP_MAX, MAP_AVG, MAP_STD, MAP_INCORE)
C
      IMPLICIT NONE
C
      include '../include/vax_minmax.inc'
C      
      INTEGER*4 NCOL, NROW, NSEC, LUN, I, J, K, SEC1, SEC2
      INTEGER*4 N1, NUMCHARI4
      REAL*4    MAP(NCOL,NROW,NSEC), MAP_MIN, MAP_MAX, MAP_AVG
      REAL*4    X, Y, SUMSQ, TOT, DIV, VAR, MAP_STD
      REAL*4    TMP(NCOL), WEIGHTS(NSEC), WEIGHT, WT1, WT2, PERCENT
      LOGICAL   MAP_INCORE
C ------------------------------------------------------------------------------
C
      IF(.NOT.MAP_INCORE) THEN
        PRINT *, '*************************************************'
        PRINT *, '* MAP not in core memory - code can not be used *'
        PRINT *, '*************************************************'
        RETURN
      ENDIF
      TOT     = 0.0
      SUMSQ   = 0.0
      PERCENT = 0.0
      DIV     = FLOAT(NCOL)*FLOAT(NROW)*FLOAT(NSEC)
C
Ca      CALL VT_CLS
      DO I = 1,NSEC
        WEIGHTS(I) = 1.0
      ENDDO
C
      WRITE(*,'(A,2F12.2,/)') ' The min/max of the MAP =',
     1                         MAP_MIN, MAP_MAX
      MAP_MIN = R4_MAX
      MAP_MAX = R4_MIN
      IF(NSEC.LE.1) THEN
 100     WRITE(*,'(A)') '$CONSTANT TO MULTIPLY VALUES BY (F): '
         READ (*,*,ERR=100) WT1
         SEC1 = 1
         SEC2 = 1
       ELSE
         PRINT *,'***********************************************'
         PRINT *,'*      ENTER SECTION NUMBERS AND WEIGHTS      *'
         PRINT *,'* ANY UNSPECIFIED SECTIONS ARE WEIGHTED = 1.0 *'
         PRINT *,'*                                             *'
         PRINT *,'* NOTE: TO REVERSE CONTRAST ENTER -1.,-1. FOR *'
         PRINT *,'*       WT1 AND WT2.                          *'
         PRINT *,'***********************************************'
         N1 = NUMCHARI4(NSEC)
 200     WRITE(*,'(A,I7,A)') '$SEC1,SEC2 (<CR>=1,', NSEC, ': '
         READ (*,*,ERR=200) SEC1, SEC2
         IF(SEC1.EQ.0 .AND. SEC2.EQ.0) THEN
           SEC1 = 1
           SEC2 = NSEC
         ENDIF
         IF(SEC1.GT.NSEC .OR. SEC2.GT.NSEC .OR. SEC1.LT.0 .OR.
     1   SEC2.LT.0 .OR. SEC1.GT.SEC2) THEN
           CALL BELL (1)
           PRINT *, '*****************************************'
           PRINT *, '* Section numbers incorrectly specified *'  
           PRINT *, '*****************************************'
           GO TO 200
         ENDIF
 300     WRITE(*,'(A)') '$WT1,WT2 (<CR>=1.0,1.0): '
         READ (*,*,ERR=300) WT1, WT2
         IF(WT1.EQ.0.0 .AND. WT2.EQ.0.0) RETURN
      ENDIF
C
      IF(SEC1.EQ.SEC2) THEN
         WEIGHTS(SEC1) = WT1
       ELSE
         DO I = SEC1,SEC2
           WEIGHTS(I) = WT1 + (I-SEC1)*(WT2-WT1)/FLOAT(SEC2-SEC1)
         ENDDO
      ENDIF
C
      DO K = 1,NSEC
        WEIGHT = WEIGHTS(K)
        DO J = 1,NROW
          DO I = 1,NCOL
            MAP(I,J,K) = MAP(I,J,K)*WEIGHT
            TMP(I)     = MAP(I,J,K)
            TOT        = TOT + TMP(I)
            SUMSQ      = SUMSQ + TMP(I)**2
            MAP_MIN    = AMIN1(MAP_MIN,TMP(I))
            MAP_MAX    = AMAX1(MAP_MAX,TMP(I))
          ENDDO
        ENDDO
      ENDDO
C
      MAP_AVG    = TOT/DIV
      MAP_STD    = 0.0
      VAR        = (SUMSQ/DIV) - (MAP_AVG**2)
      IF(VAR.GE.0.0) MAP_STD = SQRT(VAR)
C
      RETURN
      END

C  ********************************************************************
C  *                                                                  *
C  *                *****  SUBROUTINE MAP_FIX_O  *****                *
C  *                                                                  *
C  *      O  OVERWRITE values inside/outside spherical/circular or    *
C  *          octant volume.                                          *
C  *              Apply Soft/Hard border to Real MAP                  *
C  *               If RAD2 <> RAD1, the border is "SOFT", with a      *
C  *               WEIGHT = 0.0 in/out to RAD2, with an exponential   *
C  *               rise up to a WEIGHT = 1.0 at RAD1.                 *
C  *               If RAD2 = RAD1, the border is "HARD", with a       *
C  *               WEIGHT = 0.0 out to RAD2 and 1.0 for all           *
C  *               higher/lower radius data.                          *
C  *                                                                  *
C  *    JULY 1987  PURDUE UNIVERSITY VAX 11/750 L. MATTERN            *
C  *    MODIFIED 10-MAY-88 BY T. BAKER                                *
C  *    MODIFIED 17-SEP-89 BY R. H. CHENG                             *
C  *    MODIFIED 27-JUL-90 BY RHC, KAD                                *
C  *    MODIFIED 16-DEC-1997 BY T. BAKER (RID OF EM.CMM).             *
C  *    LAST MODIFIED 27-FEB-1998 BY T. BAKER (Make UNIX compatible). *
C  ********************************************************************
C
      SUBROUTINE MAP_FIX_O (LUN, MAP, NCOL, NROW, NSEC, WEIGHTS, TMP,
     1 MAP_MIN, MAP_MAX, MAP_AVG, MAP_STD, ALP, BET, GAM, MAP_INCORE)
C
      IMPLICIT NONE
C
      include '../include/vax_minmax.inc'
C      
      INTEGER*2   IX1, IX2, COUNTER
      INTEGER*4   NCOL, NROW, NSEC, LUN, I, J, K, IX, IY, IZ
      INTEGER*4   INOUT, IOPT, SEC1, SEC2, N, NUMCHARI4
      REAL*4      MAP(NCOL,NROW,NSEC), TMP(NCOL), WEIGHTS(NSEC)
      REAL*4      MAP_MIN, MAP_MAX, MAP_AVG, MAP_STD, ALP, BET, GAM
      REAL*4      X_CEN, Y_CEN, Z_CEN, XCEN, YCEN, ZCEN
      REAL*4      X, Y, Z, DX, DY, DZ, D, R
      REAL*4      COSALP, COSBET, COSGAM, SINALP, SINBET, SINGAM
      REAL*4      DENS, LOW, HIGH, SUMSQ, TOT, DIV, VAR
      REAL*4      WEIGHT, WT1, WT2, RAD2, RAD1, RAD_TMP
      REAL*4      YSQ, RAD2_SQ, RAD1_SQ, DELSQ      
      real*8      deg_to_rad
      LOGICAL     MAP_INCORE, ERROR
      CHARACTER*1 IKEY
      SAVE        COUNTER, X_CEN, Y_CEN, Z_CEN
      DATA        COUNTER/0/
C ------------------------------------------------------------------------------
C
      deg_to_rad = acos(-1.0d0)/180.0d0

      IF(.NOT.MAP_INCORE) THEN
        PRINT *, '*************************************************'
        PRINT *, '* MAP not in core memory - code can not be used *'
        PRINT *, '*************************************************'
        RETURN
      ENDIF
      COUNTER = COUNTER + 1
      IF(COUNTER.EQ.1) THEN
         X_CEN = FLOAT(NCOL+1)/2
         Y_CEN = FLOAT(NROW+1)/2
         Z_CEN = FLOAT(NSEC+1)/2
      ENDIF
C
      TOT     = 0.0
      SUMSQ   = 0.0
      DIV     = FLOAT(NCOL)*FLOAT(NROW)*FLOAT(NSEC)
C
      MAP_MIN = R4_MAX
      MAP_MAX = R4_MIN
      PRINT *,'**********************************************'
      PRINT *,'* WARNING: THIS ROUTINE PRESENTLY ONLY WORKS *'
      PRINT *,'*            WITH AN ORTHOGONAL MAP          *'
      PRINT *,'**********************************************'
      PRINT *
C
 100  IF(NSEC.GT.1) THEN
         WRITE(*,'(''$SPHERICAL <CR> OR OCTANT (1) REGION: '')')
         READ (*,*,ERR=100) IOPT
      ELSE
         IOPT = 0
      ENDIF
C
 101  IF(IOPT.NE.1) THEN
         IF(NSEC.GT.1) THEN
            WRITE(*,1000) X_CEN, Y_CEN, Z_CEN
         ELSE
            WRITE(*,1010) X_CEN, Y_CEN
         ENDIF
      ELSE
         IF(NSEC.GT.1) THEN
            WRITE(*,1020) X_CEN, Y_CEN, ZCEN
         ENDIF
      ENDIF
 1000 FORMAT('$X,Y,Z CENTER FOR SPHERE (<CR>=',F5.1,', ',F5.1,', ',
     1 F5.1,'):') 
 1010 FORMAT('$X,Y CENTER FOR CIRCLE (<CR>=',F5.1,', ',F5.1,'):') 
 1020 FORMAT('$X,Y,Z CENTER FOR OCTANT (<CR>=',F5.1,', ',F5.1,', ',
     1     F5.1,'):') 
         IF(NSEC.GT.1) THEN
            READ (*,*,ERR=101) X,Y,Z
            IF(X.LE.0.AND.Y.LE.0.AND.Z.LE.0.) THEN
              X = X_CEN            
              Y = Y_CEN
              Z = Z_CEN 
            ENDIF
            X_CEN = X
            Y_CEN = Y
            Z_CEN = Z
            X     = X - 1.
            Y     = Y - 1.
            Z     = Z - 1.
          ELSE
            READ (*,*,ERR=101) X,Y
            IF(X.LE.0.AND.Y.LE.0.) THEN
              X = X_CEN            
              Y = Y_CEN
            ENDIF
            X_CEN = X
            Y_CEN = Y
            Z_CEN = 0.
            X = X - 1.
            Y = Y - 1.
            Z = 0.
         ENDIF
C
 102     IF(IOPT.NE.1) THEN
 103        IF(NSEC.GT.1) THEN
               WRITE(*,'(''$REPLACE DENSITIES INSIDE<CR> OR
     1              OUTSIDE(1) 1 SPHERE? '')')
            ELSE
               WRITE(*,'(''$REPLACE DENSITIES INSIDE<CR> OR
     1              OUTSIDE(1) CIRCLE? '')')
            ENDIF
            READ (*,*,ERR=103) INOUT
C
      PRINT *, '******************************************************'
      PRINT *, '*     Apply Soft/Hard border to Real MAP             *'
      PRINT *, '*     ----------------------------------             *'
      PRINT *, '*                                                    *'
      PRINT *, '*      Enter values for Soft-cut (RAD2)              *'
      PRINT *, '*                                                    *'
      PRINT *, '* If RAD2 <> RAD1, the border is "SOFT", with a      *'
      PRINT *, '*  WEIGHT = 0.0 in/out to RAD2, with an exponential  *'
      PRINT *, '*  rise up to a WEIGHT = 1.0 at RAD1.                *'
      PRINT *, '*                                                    *'
      PRINT *, '* If RAD2 = RAD1, the border is "HARD", with a       *'
      PRINT *, '*  WEIGHT = 0.0 out to RAD2 and 1.0 for all          *'
      PRINT *, '*  higher/lower radius data.                         *'
      PRINT *, '******************************************************'
      PRINT * 
C
      DENS = 0.
      IF(NSEC.GT.1) THEN
         WRITE(*,'(''$DENSITY AND RADIUS OF SPHERE TO CUT
     1 DENS, RAD1, RAD2 (3F):'')')
      ELSE
         WRITE(*,'(''$DENSITY AND RADIUS OF CIRCLE TO CUT
     1 DENS, RAD1, RAD2 (3F):'')')
      ENDIF
 104  IF(INOUT.NE.1) THEN       ! THE INSIDE FIX
 910     PRINT *,'****************************************************'
         PRINT *,'*    RAD2 must be less than RAD1 for Soft Border,  *'
         PRINT *,'*         or equal RAD1 for Hard Border            *'
         PRINT *,'****************************************************'
         READ (*,*,ERR=910) DENS, RAD1, RAD2 
             IF(RAD2.LE.0.) RAD2 = RAD1
        N = NUMCHARI4(NINT(DENS)) + 3
        IF(RAD2.GT.RAD1) GOTO 910      
C
         IF(RAD1.EQ.RAD2) THEN                  ! HARD-CUT IN
           WRITE(*,'(A,F8.2,A,F5.1)') ' HARD-CUT DENSITY: ',
     1     DENS, ' at ', RAD2
              ELSE                                   ! SOFT-CUT OUT
                WRITE(*,'(A,F8.2,A,F5.1,A,F5.1)') ' SOFT-CUT DENSITY:',
     1     DENS, ' at BORDER:', RAD1, ', ', RAD2
              ENDIF
C  ----------
C                              ! THE OUT-SPHERE FIX
           ELSEIF(INOUT.EQ.1) THEN
 950          PRINT *,'***********************************************'
              PRINT *,'*      RAD2 must be larger than RAD1 for      *'
              PRINT *,'*  Soft Border or equal RAD1 for Hard Border  *'
              PRINT *,'***********************************************'
        READ (*,*,ERR=950) DENS, RAD1, RAD2 
             IF(RAD2.LE.0.) RAD2 = RAD1
        N = NUMCHARI4(NINT(DENS)) + 3
        IF(RAD2.LT.RAD1) GOTO 950      
C
         IF(RAD1.EQ.RAD2) THEN                   ! HARD-CUT OUT
           WRITE(*,'(A,F8.2,A,F5.1)') ' HARD-CUT DENSITY: ',
     1     DENS, ' at ', RAD2
              ELSE                                   ! SOFT-CUT OUT
                WRITE(*,'(A,F8.2,A,F5.1,A,F5.1)') ' SOFT-CUT DENSITY:',
     1     DENS, ' at BORDER:', RAD1, ', ', RAD2
              ENDIF                        !  CLOSE INOUT-loop
          ENDIF
C  ------------------                                     
       ELSE                              !  OCTANT 
         WRITE(*,'(A)') '$DENSITY VALUE INSIDE OCTANT (F): '
       ENDIF                        !  CLOSE IOPT-loop      
C
         N = NUMCHARI4 (NSEC)
 105     WRITE(*,'(A,I8,A)') '$RANGE OF SECTIONS TO INCLUDE (<CR>=1,',
     1        NSEC, '): '
         READ (*,*,ERR=105) SEC1, SEC2
         IF(SEC1.GT.SEC2) THEN
           N    = SEC1
           SEC1 = SEC2
           SEC2 = N
         ENDIF
         IF(SEC1.EQ.0 .AND. SEC2.EQ.0) THEN
           SEC1 = 1
           SEC2 = NSEC
         ENDIF
         IF(SEC1.LT.0.OR.SEC1.GT.NSEC.OR.SEC2.LT.0.OR.
     1   SEC2.GT.NSEC) THEN
          CALL BELL (1)
          GO TO 105
         ENDIF
C
         COSALP  = COS(ALP*deg_to_rad)
         COSBET  = COS(BET*deg_to_rad)
         COSGAM  = COS(GAM*deg_to_rad)
         SINALP  = SIN(ALP*deg_to_rad)
         SINBET  = SIN(BET*deg_to_rad)
         SINGAM  = SIN(GAM*deg_to_rad)
         XCEN    = X + Y*COSGAM + Z*COSBET
         YCEN    =     Y*SINGAM + Z*COSALP
         ZCEN    =                Z*SINALP*SINBET
         RAD2_SQ = RAD2*RAD2
         RAD1_SQ = RAD1*RAD1
         DELSQ   = (RAD1-RAD2)**2
C 
         IF(NSEC.LE.1) THEN                                     ! 2D MAP
            DO J = 1,NROW
              IY = J - 1
              Y  = IY*SINGAM
              DY = (Y-YCEN)**2
              DO I = 1,NCOL
                IX = I - 1
                X  = IX + IY*COSGAM
                DX = (X-XCEN)**2
                D  = DX + DY
                TMP(I) = MAP(I,J,1)
C -->   THE IN-CUT
                IF(D.LE.RAD1_SQ) THEN
                  IF(INOUT.NE.1) THEN
                         IF(DELSQ.EQ.0.0) THEN             ! HARD-CUT
                      TMP(I) = DENS
                          ELSE 
                         IF(D.LT.RAD2_SQ) THEN
                        TMP(I) = DENS
                          ELSE
                              WEIGHT = 1.0-EXP(-(6.0*
     1                             (SQRT(D)-RAD2)**2/DELSQ))
C                          WRITE(16,'(3I4,F10.3)') I,J,1,WEIGHT
                          IF(DENS.NE.0.) MAP(I,J,1)=MAP(I,J,1)+DENS
                         TMP(I) = MAP(I,J,1) * WEIGHT
                      ENDIF
                       ENDIF
                  ENDIF
C --->  THE OUT-CUT
                ELSE
                  IF(D.GE.RAD1_SQ) THEN
                    IF(INOUT.EQ.1) THEN 
                         IF(DELSQ.EQ.0.0) THEN             ! HARD-CUT
                         TMP(I) = DENS
                     ELSE 
                         IF(D.GT.RAD2_SQ) THEN 
                           TMP(I) = DENS 
                           ELSE
                               WEIGHT = 1.0-EXP(-(6.0*
     1                             (SQRT(D)-RAD2)**2/DELSQ))
                           IF(DENS.NE.0.) MAP(I,J,1)=MAP(I,J,1)+DENS
                          TMP(I) = MAP(I,J,1) * WEIGHT
                       ENDIF
                       ENDIF
                    ENDIF
                     ENDIF
                ENDIF
C
                TOT        = TOT + TMP(I)
                SUMSQ      = SUMSQ + TMP(I)**2
                MAP_MIN    = AMIN1(MAP_MIN,TMP(I))
                MAP_MAX    = AMAX1(MAP_MAX,TMP(I))
                MAP(I,J,1) = TMP(I)
              ENDDO
            ENDDO
C
          ELSE                                                  ! 3D MAP
C     OPEN(16,STATUS='NEW',FILE='WEIGHTS')
            DO K = SEC1,SEC2
              IZ = K - 1
              Z  = IZ*SINALP*SINBET
              DZ = (Z-ZCEN)**2
              DO J = 1,NROW
                IY = J - 1
                Y  = IY*SINGAM + IZ*COSALP
                DY = (Y-YCEN)**2
                DO I = 1,NCOL
                  TMP(I) = MAP(I,J,K)
                  IX     = I - 1
                  X      = IX + IY*COSGAM + IZ*COSBET
C -->   THE IN-CUT
                  IF(IOPT.NE.1) THEN             
                     DX = (X-XCEN)**2
                     D  = DX + DY + DZ
                     IF(D.LE.RAD1_SQ) THEN
                        IF(INOUT.NE.1) THEN
                            IF(DELSQ.EQ.0.0) THEN       ! HARD-CUT
                            TMP(I) = DENS
                             ELSE 
                              IF(D.LT.RAD2_SQ) THEN
                             TMP(I) = DENS
                               ELSE
                                   WEIGHT = 1.0-EXP(-(6.0*
     1                             (SQRT(D)-RAD2)**2/DELSQ))
                               IF(DENS.NE.0.) THEN 
                                MAP(I,J,K)=MAP(I,J,K)+DENS
                               ENDIF
                              TMP(I) = MAP(I,J,K) * WEIGHT
                           ENDIF
                            ENDIF
                          MAP(I,J,K) = TMP(I)
                        ENDIF
C --->  THE OUT-CUT
                     ELSE
                        IF(D.GE.RAD1_SQ) THEN
                          IF(INOUT.EQ.1) THEN
                              IF(DELSQ.EQ.0.0) THEN       ! HARD-CUT
                              TMP(I) = DENS
                               ELSE 
                                 IF(D.GT.RAD2_SQ) THEN
                                TMP(I) = DENS
                                  ELSE
                                      WEIGHT = 1.0-EXP(-(6.0*
     1                             (SQRT(D)-RAD2)**2/DELSQ))
                                  IF(DENS.NE.0.) THEN 
                                  MAP(I,J,K)=MAP(I,J,K)+DENS
                                  ENDIF
                                 TMP(I) = MAP(I,J,K) * WEIGHT
                              ENDIF
                              ENDIF
                            MAP(I,J,K) = TMP(I)
                          ENDIF
                         ENDIF
                        ENDIF
                  ELSE
                     IF(Z.GT.ZCEN) THEN
                       IF((Z-ZCEN).GT.ABS(Y-YCEN)) THEN
                         IF((ABS((Y-YCEN)-(Z-ZCEN))/2.0).GT.
     1                       ABS(X-XCEN)) THEN
                             TMP(I)     = DENS
                             MAP(I,J,K) = TMP(I)
                         ENDIF
                       ENDIF
                     ENDIF
                  ENDIF
                  TOT     = TOT + TMP(I)
                  SUMSQ   = SUMSQ + TMP(I)**2
                  MAP_MIN = AMIN1(MAP_MIN,TMP(I))
                  MAP_MAX = AMAX1(MAP_MAX,TMP(I))
                ENDDO
              ENDDO
            ENDDO
         ENDIF
         MAP_AVG = TOT/DIV
         MAP_STD = 0.0
         VAR     = (SUMSQ/DIV) - (MAP_AVG**2)
         IF(VAR.GE.0.0) MAP_STD = SQRT(VAR)
C
      RETURN
      END

C  *********************************************************************
C  *                                                                   *
C  *                  *****  SUBROUTINE MAP_FIX_R  *****               *
C  *                                                                   *
C  *    Recale one density or a range of densities.                    *
C  *                                                                   *
C  *    AUGUST 1990  PURDUE UNIVERSITY VAX 8550 T. BAKER               *
C  *      Modified screen output, 22 Sept 1994, by D. Belnap             *
C  *    MODIFIED 16-DEC-1997 BY T. BAKER (Rid of EM.CMM).              *
C  *    LAST MODIFIED 27-FEB-1998 BY T. BAKER (Make UNIX compatible).  *
C  *********************************************************************
C
      SUBROUTINE MAP_FIX_R (LUN, MAP, NCOL, NROW, NSEC, TMP, MAP_MIN,
     1 MAP_MAX, MAP_AVG, MAP_STD, MAP_SCALEF, FILE_ID, MAP_INCORE)
C
      IMPLICIT NONE
C
      INTEGER*4 NCOL, NROW, NSEC, LUN, FILE_ID
      REAL*4    MAP(NCOL,NROW,NSEC), TMP(NCOL), TMP1, TMP2
      REAL*4    MAP_MIN, MAP_MAX, MAP_AVG, MAP_STD, MAP_SCALEF
      REAL*4    DENS1, DENS2, DENS3, DENS4
      LOGICAL   MAP_INCORE
C ------------------------------------------------------------------------------
C
      IF(.NOT.MAP_INCORE) THEN
        PRINT *, '*************************************************'
        PRINT *, '* MAP not in core memory - code can not be used *'
        PRINT *, '*************************************************'
        RETURN
      ENDIF
C
      DENS1 = MAP_MIN
      DENS2 = MAP_MAX
      PRINT *, '**********************************************'
      PRINT *, '* SPECIFY RANGE (LOW-HIGH) OF DENSITY VALUES *'
      PRINT *, '*  FOR RESCALING.                            *'
      PRINT *, '*                                            *'
      PRINT *, '* DEFAULT RANGE = MAP_MIN,MAP_MAX            *'
      PRINT *, '* NOTE: Current MAP_AVG =', MAP_AVG,   '     *'
      PRINT *, '**********************************************'
      PRINT *, 'MAP_MIN = ', MAP_MIN
      PRINT *, 'MAP_MAX = ', MAP_MAX
  100      WRITE(*,'(A)') '$DENS1,DENS2 (<CR>=MAP_MIN,MAP_MAX): '
      READ (*,*,ERR=100) TMP1, TMP2
      IF(TMP1.NE.0.0 .OR. TMP2.NE.0.0) THEN
        DENS1 = MIN(TMP1,TMP2)
        DENS2 = MAX(TMP1,TMP2)
      ENDIF
C
  200      WRITE(*,'(A)') '$NEW RANGE (<CR>=MAP_MIN,MAP_MAX): '
      READ (*,*,ERR=200) DENS3, DENS4
      IF(DENS3.EQ.0.0 .AND. DENS4.EQ.0.0) RETURN
C
      CALL MAP_SCALE (LUN, MAP, NCOL, NROW, NSEC, DENS1, DENS2,
     1 DENS3, DENS4, MAP_MIN, MAP_MAX, MAP_AVG, MAP_STD, MAP_SCALEF, 
     2 FILE_ID, MAP_INCORE)
C
      RETURN
      END

C  ****************************************************************************
C
      SUBROUTINE MAP_GET_ANGLE (T_MATRIX, A_CELL, B_CELL, C_CELL,
     1 MAP_ABANG, ALP, BET, GAM)
C
      IMPLICIT NONE
C
      INTEGER*4 T_MATRIX(2,3)
      REAL*4    V1, V2, A_CELL, B_CELL, C_CELL, MAP_ABANG
      REAL*4    ALP, BET, GAM
      real*4    cosalp, cosbet, cosgam
      real*8    deg_to_rad, rad_to_deg
C  ----------------------------------------------------------------------------
C
      deg_to_rad = acos(-1.0d0)/180.0d0
      rad_to_deg = 1.0d0/deg_to_rad
      cosalp = cos(deg_to_rad*alp)
      cosbet = cos(deg_to_rad*bet)
      cosgam = cos(deg_to_rad*gam)
      
      V1 = SQRT( (T_MATRIX(1,1)*A_CELL)**2 + 
     1          (T_MATRIX(1,2)*B_CELL)**2 +
     2          (T_MATRIX(1,3)*C_CELL)**2 )
      V2 = SQRT( (T_MATRIX(2,1)*A_CELL)**2 +
     1          (T_MATRIX(2,2)*B_CELL)**2 +
     2          (T_MATRIX(2,3)*C_CELL)**2 )
      MAP_ABANG = (T_MATRIX(1,1)*T_MATRIX(2,1)*A_CELL*A_CELL) +
     1       (T_MATRIX(1,1)*T_MATRIX(2,2)*A_CELL*B_CELL*COSGAM) +
     2       (T_MATRIX(1,1)*T_MATRIX(2,3)*A_CELL*C_CELL*COSBET) +
     3       (T_MATRIX(1,2)*T_MATRIX(2,2)*B_CELL*B_CELL) +
     4       (T_MATRIX(1,2)*T_MATRIX(2,1)*A_CELL*B_CELL*COSGAM) +
     5       (T_MATRIX(1,2)*T_MATRIX(2,3)*B_CELL*C_CELL*COSALP) +
     6       (T_MATRIX(1,3)*T_MATRIX(2,3)*C_CELL*C_CELL) +
     7       (T_MATRIX(1,3)*T_MATRIX(2,1)*A_CELL*C_CELL*COSBET) +
     8       (T_MATRIX(1,3)*T_MATRIX(2,2)*B_CELL*C_CELL*COSALP)
      MAP_ABANG = rad_to_deg * ACOS(MAP_ABANG/(V1*V2) )
C
      RETURN
      END

C  ***************************************************************************
C  *                                                                         *
C  *                  *****     SUBROUTINE MAP_IMG     *****                 *
C  *                                                                         *
C  *    CONVERT "MAP" DATA TO "IMAGE" DATA.                                  *
C  *                                                                         *
C  *    DECEMBER 1985  PURDUE UNIVERSITY VAX 11/750, T. BAKER.               *
C  *    MODIFIED 12-DEC-1997  BY T. BAKER.                                   *
C  ***************************************************************************
C
      SUBROUTINE MAP_IMG (IMG, MAP, NCOL, NROW, NSEC, SECTION, MAP_MIN,
     1 MAP_MAX)
C
      IMPLICIT NONE
C
      INTEGER*4 NCOL, NROW, NSEC, SECTION
      real*4    IMG(NCOL,NROW)
      INTEGER*4 I, J, I_MIN, I_MAX
      REAL*4    MAP(NCOL,NROW,NSEC), MAP_MIN, MAP_MAX, SCALE
      LOGICAL   YES_SCALE
C  ----------------------------------------------------------------------------
C
C  ****************************************************************
C  *  NOTE, FOR NOW, THE VALUES ARE AUTOMATICALLY SCALED BETWEEN  *
C  *   I_MIN-I_MAX IF MAP_MIN & MAP_MAX ARE OUT OF RANGE          *
C  ****************************************************************
      PRINT *,'MAP_MIN, MAP_MAX = ', MAP_MIN,MAP_MAX
  50      WRITE(*,'(2A)') '$Enter IMAGE data minimum and maximum (2I) ',
     1        '(<CR>=0,255):  '
      READ (*,*,ERR=50) I_MIN, I_MAX
      IF (I_MAX.EQ.0)  I_MAX = 255
      IF(MAP_MIN.LT.FLOAT(I_MIN).OR.MAP_MAX.GT.FLOAT(I_MAX)) THEN
         YES_SCALE = .TRUE.
         PRINT *,'------------------------------------------------'
         PRINT *,'Your MAP data lie outside the IMAGE min. or max.'
         PRINT *,'The data will be automatically scaled'
         PRINT *,'------------------------------------------------'
       ELSE
         PRINT *,'------------------------------------------------'
         PRINT *,'Your MAP data lie within the IMAGE min. and max.'
         WRITE(*,'(A)') '$Do you want to rescale your data? (Y/<N>) '
         CALL GET_YESNO(YES_SCALE, .FALSE.)
         PRINT *,'------------------------------------------------'
      ENDIF
      IF (YES_SCALE) THEN
         PRINT *, 'DATA SCALED TO IMAGE MIN. & MAX.'
         SCALE = FLOAT(I_MAX-I_MIN)/(MAP_MAX-MAP_MIN)
         DO J = 1,NROW
           DO I = 1,NCOL
             IMG(I,J) = NINT(SCALE*(MAP(I,J,SECTION)-MAP_MIN))
           ENDDO
         ENDDO
       ELSE
         PRINT *, 'DATA NOT SCALED'
         DO J = 1,NROW
           DO I = 1,NCOL
             IMG(I,J) = NINT(MAP(I,J,SECTION))
           ENDDO
         ENDDO
      ENDIF
C
      RETURN
      END

C  ****************************************************************************
C  *                                                                          *
C  *               *****     SUBROUTINE MAP_MULTIPLY     *****                *
C  *                                                                          *
C  *    MULTIPLY MAP DATA BY A CONSTANT SCALE FACTOR.                         *
C  *                                                                          *
C  *    MAY 1989  PURDUE UNIVERSITY VAX 8550, T. BAKER.                       *
C  *    LAST MODIFIED 12-DEC-1997 BY T. BAKER.                                *
C  ****************************************************************************
C
      SUBROUTINE MAP_MULTIPLY (MAP, NCOL, NROW, NSEC, SCALE, MAP_MIN,
     1 MAP_MAX, MAP_AVG, MAP_STD)
C
      IMPLICIT NONE
C
      INTEGER*4 I, NCOL, NROW, NSEC, ARRAY_SIZE
      REAL*4    MAP(1), SCALE, MAP_MIN, MAP_MAX, MAP_AVG, MAP_STD
C  ----------------------------------------------------------------------------
C
      ARRAY_SIZE = NCOL*NROW*NSEC
C
      DO I = 1,ARRAY_SIZE
        MAP(I) = MAP(I)*SCALE
      ENDDO
C
      MAP_MIN = MAP_MIN*SCALE
      MAP_MAX = MAP_MAX*SCALE
      MAP_AVG = MAP_AVG*SCALE
      MAP_STD = MAP_STD*SCALE
C
      RETURN
      END

C  ********************************************************************
C  *                                                                  *
C  *              *****     SUBROUTINE MAP_NOISE     *****            *
C  *                                                                  *
C  *    ROUTINE TO ADD RANDOM NOISE TO "MAP"                          *
C  *                                                                  *
C  *    APRIL 1984  PURDUE UNIVERSITY VAX 11/750, T. BAKER.           *
C  *    MAY 1988  PURDUE UNIVERSITY VAX 8550, T. BAKER.               *
C  *    MODIFIED 16-DEC-1997 BY T. BAKER (Rid of EM.CMM).             *
C  *    LAST MODIFIED 27-FEB-1998 BY T. BAKER (Make UNIX compatible). *
C  ********************************************************************
C
      SUBROUTINE MAP_NOISE (LUN, MAP, NCOL, NROW, NSEC, TMP, NOISE, 
     1 MAP_MIN, MAP_MAX, MAP_AVG, MAP_STD, MAP_INCORE)
C
      IMPLICIT NONE
C
      include '../include/vax_minmax.inc'
C
      INTEGER*4 LUN, NCOL, NROW, NSEC, ISEED, I, J, K
      REAL*4    MAP(NCOL,NROW,NSEC), TMP(NCOL), NOISE, X
      REAL*4    PERCENT, SUMSQ, TOT, DIV, VAR, RAN
      REAL*4    MAP_MIN, MAP_MAX, MAP_AVG, MAP_STD
      LOGICAL   MAP_INCORE
C  ----------------------------------------------------------------------------
C
      IF(.NOT.MAP_INCORE) THEN
        PRINT *, '*************************************************'
        PRINT *, '* MAP not in core memory - code can not be used *'
        PRINT *, '*************************************************'
        RETURN
      ENDIF
      IF(NSEC.LE.0) THEN
        PRINT *, '*************************'
        PRINT *, '* NSEC is .LE. 0        *'
        PRINT *, '* It has been reset = 1 *'
        PRINT *, '*************************'
        NSEC = 1
      ENDIF
C
      MAP_MIN = R4_MAX
      MAP_MAX = R4_MIN
      SUMSQ   = 0.0
      TOT     = 0.0
      DIV     = FLOAT(NCOL)*FLOAT(NROW)*FLOAT(NSEC)
C
      CALL SEED (ISEED)
C
      IF(NSEC.LE.1) THEN
C          ****************
C----------*    2D MAP    *
C          ****************
         DO J = 1,NROW
           DO I = 1,NCOL
             X          = MAP(I,J,1) + (RAN(ISEED)-.5)*2.0*NOISE
             MAP(I,J,1) = X
             SUMSQ      = SUMSQ + X**2
             TOT        = TOT + X
             MAP_MIN    = MIN(MAP_MIN,X)
             MAP_MAX    = MAX(MAP_MAX,X)
           ENDDO
         ENDDO
       ELSE
C          ****************
C----------*    3D MAP    *
C          ****************
         DO K = 1,NSEC
           DO J = 1,NROW
             DO I = 1,NCOL
               X          = MAP(I,J,K) + (RAN(ISEED)-.5)*2.0*NOISE
               MAP(I,J,K) = X
               SUMSQ      = SUMSQ + X**2
               TOT        = TOT + X
               MAP_MIN    = MIN(MAP_MIN,X)
               MAP_MAX    = MAX(MAP_MAX,X)
             ENDDO
           ENDDO
         ENDDO
      ENDIF
C
      MAP_AVG = TOT/DIV
      MAP_STD = 0.0
      VAR     = (SUMSQ/DIV)-(MAP_AVG*MAP_AVG)
      IF(VAR.GE.0.0) MAP_STD = SQRT(VAR)
C
      RETURN
      END

C  **************************************************************************
C  *                                                                        *
C  *                *****     SUBROUTINE MAP_PEAK     *****                 *
C  *                                                                        *
C  *    ROUTINE TO SEARCH FOR THE HIGHEST PEAK IN A MAP SECTION.            *
C  *    NOTE THAT THIS ROUTINE ONLY WORKS AT PRESENT WITH 2D MAP ARRAYS.    *
C  *                                                                        *
C  *    IPEAKX = COLUMN POSITION OF "PEAK"                                  *
C  *    IPEAKY = ROW POSITION OF "PEAK"                                     *
C  *    PEAK   = INTENSITY OF "PEAK"                                        *
C  *                                                                        *
C  *    MAY 1992  PURDUE UNIVERSITY VAX 8550, T. BAKER.                     *
C  *    LAST MODIFIED 28-MAY-1992 BY T. BAKER.                              *
C  **************************************************************************
C
      SUBROUTINE MAP_PEAK (MAP, NCOL, NROW, IPEAKX, IPEAKY, PEAK)
C
      IMPLICIT NONE
C
      include '../include/vax_minmax.inc'
C
      INTEGER*4 I, J, IPEAKX, IPEAKY, NCOL, NROW
      REAL*4    PEAK, MAP(NCOL,NROW)
C  ----------------------------------------------------------------------------
C
      PEAK = R4_MIN
C
      DO J = 1,NROW
        DO I = 1,NCOL
          IF(MAP(I,J).GT.PEAK) THEN
            IPEAKX = I
            IPEAKY = J
            PEAK   = MAP(I,J)
          ENDIF
        ENDDO
      ENDDO
C
      RETURN
      END

C  **************************************************************************
C  *                                                                        *
C  *              *****     SUBROUTINE MAP_PEAKSEARCH     *****             *
C  *                                                                        *
C  *    ROUTINE TO SEARCH FOR ALL NON-ADJACENT "MAP" VALUES ABOVE LOWLIM.   *
C  *    NOTE THAT THIS ROUTINE ONLY WORKS AT PRESENT WITH 2D MAP ARRAYS.    *
C  *                                                                        *
C  *    PEAKS(1,NPEAK) = COLUMN POSITION OF "PEAK"                          *
C  *    PEAKS(2,NPEAK) = ROW POSITION OF "PEAK"                             *
C  *    PEAKS(3,NPEAK) = INTENSITY OF "PEAK"                                *
C  *                                                                        *
C  *    MAY 1989  PURDUE UNIVERSITY VAX 8550, T. BAKER                      *
C  *    LAST MODIFIED 16-DEC-1997 BY T. BAKER (RID OF EM.CMM).              *
C  **************************************************************************
C
      SUBROUTINE MAP_PEAKSEARCH (MAP, NCOL, NROW, PEAKS, NLIM, NPEAK,
     1                          LOWLIM)
C
      IMPLICIT NONE
C
      INTEGER*4 NLIM, NPEAK, COL, ROW, N1, N2, NUMCHARI4
      INTEGER*4 I, NCOL, NROW, J_LAST, J_NEXT
      REAL*4    PEAKS(3,NLIM), MAP(NCOL,NROW), TEST, LOWLIM
C  ----------------------------------------------------------------------------
C
      DO I = 1,NLIM
        PEAKS(1,I) = 0.0
        PEAKS(2,I) = 0.0
        PEAKS(3,I) = 0.0
      ENDDO
C
      N1 = NUMCHARI4(NINT(LOWLIM)) + 2
Ca      WRITE(*,'(2A,F<N1>.1,A)') ' # of non-adjacent pixels',
Ca      1 ' with intensities > ', LOWLIM, ' = Wait for answer'
C
      NPEAK = 0                        ! RESET NPEAK COUNTER
C
      DO ROW = 1,NROW                        ! STEP THROUGH ROW BY ROW
        J_LAST = ROW - 1
        J_NEXT = ROW + 1
        DO COL = 1,NCOL                  ! STEP THROUGH CURRENT ROW
          TEST = MAP(COL,ROW)
          IF(TEST.GT.LOWLIM) THEN
            IF(COL.LT.NCOL) THEN
              IF(TEST.LE.MAP(COL+1,ROW)) GO TO 100
            ENDIF
            IF(COL.GT.1) THEN
              IF(TEST.LT.MAP(COL-1,ROW)) GO TO 100
            ENDIF
            IF(J_NEXT.LE.NROW) THEN
              IF(TEST.LE.MAP(COL,J_NEXT)) GO TO 100
            ENDIF
            IF(J_LAST.GT.0) THEN
              IF(TEST.LT.MAP(COL,J_LAST)) GO TO 100
            ENDIF
            NPEAK = NPEAK + 1
            IF(NPEAK.GT.NLIM) THEN            ! TRAP FOR ARRAY OVERFLOW
              NPEAK = -1                        ! SIGNAL OVERFLOW
              CALL BELL (3)
              PRINT *, '***************************************'
              PRINT *, '* TOO MANY PEAK PIXELS - RESET LOWLIM *'
              PRINT *, '***************************************'
              RETURN
            ENDIF
            PEAKS(1,NPEAK) = FLOAT(COL)
            PEAKS(2,NPEAK) = FLOAT(ROW)
            PEAKS(3,NPEAK) = MAP(COL,ROW)
          ENDIF
  100       CONTINUE
        ENDDO
      ENDDO
C
      N2 = NUMCHARI4(NPEAK)
Ca      WRITE(*,'(2A,F<N1>.1,A,I<N2>,A)') '+# of non-adjacent pixels',
Ca      1 ' with intensities > ', LOWLIM, ' = ', NPEAK, '            '
C
      RETURN
      END


C  ****************************************************************************
C  *                                                                          *
C  *                  *****     SUBROUTINE MAP_ROT     *****                  *
C  *                                                                          *
C  *    ROTATE 2D OR 3D MAP ABOUT ITS CENTER BY "RANGL" DEGREES.              *
C  *                                                                          *
C  *    MAY 1990  PURDUE UNIVERSITY VAX 8550 BY WDG, RHC.                     * 
C  *    CAPABILITY TO ROTATE MAP ABOUT X,Y,Z-AXIS ADDED.                      *
C  *                                                                          *
C  *    POINT LAYOUT FOR BI-LINEAR INTERPOLATION SCHEME :                     *
C  *                                                                          *
C  *          D4            D3                                                *
C  *                                                                          *
C  *                    + (dx,dy)                                             *
C  *                    .                                                     *
C  *          D1. . . . .   D2                                                *
C  *                                                             *
C  *    THE INTERPOLATION ALGORITHM IS COMPUTED AS FOLLOWS:                   *
C  *                                                                          *
C  *          D1(1-dX)(1-dY) + D2(dX)(1-dY) + D3(dX)(dY) + D4(1-dX)(dY)       *
C  *                                                                          *
C  *    WHICH REDUCES TO:                                                     *
C  *                                                                          *
C  *          D1 + (D2-D1)dx + (D4-D1)dy + (D1-D2+D3-D4)dxdy                  *
C  *                                                             *
C  *    AUGUST 1989  PURDUE UNIVERSITY VAX 8550, T. BAKER.                    *
C  *    MODIFIED 11-DEC-1997 BY T. BAKER.                                     *
C  *    LAST MODIFIED 25-FEB-1998 BY T. BAKER (Add UNIX compatibility).       *
C  ****************************************************************************
C
      SUBROUTINE MAP_ROT (MAP1, NCOL, NROW, NSEC, RANGL, AXIS)
C
      IMPLICIT NONE
C
      INTEGER*4 NCOL, NROW, NSEC
      REAL*4    MAP1(NCOL, NROW, NSEC), RANGL
      CHARACTER AXIS*1
      real*4, allocatable :: map2(:), tmp(:)
      integer   alloc_stat
      LOGICAL   PHANTOM
C  ----------------------------------------------------------------------------
C
C  ***********************************************************
C  *    NOTE TO PROGRAMMERS:                                 *
C  *     THE FOLLOWING PHANTOM CAN BE USED (IF .TRUE.) IF    *
C  *     YOU WANT TO CHECK THE SCRATCH DISK VERSION OF THE   *
C  *     CODE WITH SMALLER 3D DATA SETS.                     *
C  ***********************************************************
      PHANTOM = .FALSE.
C
      IF(AXIS.EQ.'Z') THEN

         allocate(map2(ncol*nrow), stat=alloc_stat)
         if(alloc_stat > 0) then
            write(*,*) 'Unable to allocate MAP2 in MAP_ROT'
            call tsbend
         endif

         CALL MAP_DOLLAR_ROT (MAP1, MAP2, NCOL, NROW, NSEC, RANGL)
         deallocate(map2)
       ELSE IF(AXIS.EQ.'X') THEN
         IF(NCOL.NE.NROW .OR. NCOL.NE.NSEC .OR. NROW.NE.NSEC) THEN
           CALL BELL (2)
           PRINT *, '*************************************'
           PRINT *, '* Only able to handle CUBIC 3D maps *'
           PRINT *, '*************************************'
           CALL HITCR
         ENDIF
         allocate(map2(nrow*nsec), tmp(nrow), stat=alloc_stat)
         if(alloc_stat > 0) then
            write(*,*) 'Unable to allocate MAP2 and TMP in MAP_ROT'
            call tsbend
         endif
         IF(.NOT.PHANTOM) THEN
            CALL MAP_ROTX (MAP1, MAP2, NCOL, NROW, NSEC, RANGL)
          ELSE
            CALL MAP_ROTX_SLOW (MAP1,MAP2,TMP,NCOL,NROW,NSEC,RANGL)
         ENDIF
         deallocate(map2, tmp)
       ELSE IF(AXIS.EQ.'Y') THEN
         IF(NCOL.NE.NROW .OR. NCOL.NE.NSEC .OR. NROW.NE.NSEC) THEN
           CALL BELL (2)
           PRINT *, '*************************************'
           PRINT *, '* Only able to handle CUBIC 3D maps *'
           PRINT *, '*************************************'
           CALL HITCR
         ENDIF
         allocate(map2(ncol*nsec), tmp(nsec), stat=alloc_stat)
         if(alloc_stat > 0) then
            write(*,*) 'Unable to allocate MAP2 and TMP in MAP_ROT'
            call tsbend
         endif
         IF(.NOT.PHANTOM) THEN
            CALL MAP_ROTY (MAP1, MAP2, NCOL, NROW, NSEC, RANGL)
          ELSE
            CALL MAP_ROTY_SLOW (MAP1,MAP2,TMP,NCOL,NROW,NSEC,RANGL)
         ENDIF
         deallocate(map2, tmp)
      ENDIF
C
      RETURN
      END
C  ============================================================================
      SUBROUTINE MAP_DOLLAR_ROT (MAP1, MAP2, NCOL, NROW, NSEC, RANGL)
C
      IMPLICIT NONE
C
      INTEGER*4 NCOL, NROW, NSEC, IX, IY, I, J, K, MOVE_SIZE, TEST
      REAL*4    XCEN, YCEN, A11, A12, A21, A22, PERCENT, DIVISOR
      REAL*4    D1, D2, D3, D4, DX, DY, X, Y, RANGL
      REAL*4    MAP1(NCOL,NROW,NSEC), MAP2(NCOL,NROW)
      real*8    deg_to_rad
      LOGICAL   NOTIFY
C  ----------------------------------------------------------------------------
C
      deg_to_rad = acos(-1.0d0)/180.0d0
      DIVISOR = 1.0
C
      TEST = NCOL*NROW*NSEC
      IF(TEST.GT.5000000) THEN
         NOTIFY = .TRUE.
         DIVISOR = FLOAT(NCOL)
         WRITE(*,'(1X, ''  0.0% complete'')')
       ELSE
         NOTIFY = .FALSE.
      ENDIF
C
      XCEN      = FLOAT(NCOL+1)/2.0
      YCEN      = FLOAT(NROW+1)/2.0
      MOVE_SIZE = 4*NCOL
C  ********************************
C  *    SET UP ROTATION MATRIX    *
C  ********************************
      A11 = COS(RANGL*deg_to_rad)
      A12 = SIN(RANGL*deg_to_rad)
      A21 = -A12
      A22 =  A11
C
      DO K = 1,NSEC                  ! STEP THROUGH NSEC SECTIONS
        DO J = 1,NROW                  ! STEP THROUGH ROWS OF MAP2
          DO I = 1,NCOL            ! STEP THROUGH CURRENT ROW OF MAP2
            X = A11*(FLOAT(I)-XCEN)+A12*(FLOAT(J)-YCEN) + XCEN
            Y = A21*(FLOAT(I)-XCEN)+A22*(FLOAT(J)-YCEN) + YCEN
            IX = X
            IY = Y
            IF(IX.GE.1.AND.IY.GE.1.AND.IX.LE.NCOL.AND.IY.LE.NROW) THEN
               D2        = 0.0
               D3        = 0.0
               D4        = 0.0
               D1        = MAP1(IX,IY,K)
               IF(IX.LT.NCOL) D2 = MAP1(IX+1,IY,K)
               IF(IX.LT.NCOL.AND.IY.LT.NROW) D3 = MAP1(IX+1,IY+1,K)
               IF(IY.LT.NROW) D4 = MAP1(IX,IY+1,K)
               DX        = X-IX
               DY        = Y-IY
               MAP2(I,J) = D1+DX*(D2-D1)+DY*(D4-D1)+DX*DY*(D1-D2+D3-D4)
             ELSE
               MAP2(I,J) = 0.0
            ENDIF
          ENDDO
        ENDDO                                    ! LOOP OVER ROWS
C  **********************************************
C  *    STORE ROTATED SECTION BACK INTO MAP1    *
C  **********************************************
        DO J = 1,NROW
          DO I = 1,NCOL
            MAP1(I,J,K) = MAP2(I,J)
CCC            CALL LIB$MOVC3 (MOVE_SIZE, MAP2(1,J), MAP1(1,J,K))
          ENDDO
        ENDDO
        IF(NOTIFY) THEN
          PERCENT = 100.0*(FLOAT(K)/DIVISOR)
          WRITE(*,'(''+'',F5.1)') PERCENT
        ENDIF
      ENDDO                                    ! LOOP OVER SECTIONS
C
      RETURN
      END

C  *************************************************************************
C  *                                                                       *
C  *              *****     SUBROUTINE MAP_ROTX     *****                  *
C  *                                                                       *
C  *    IN CORE ROTATION OF 3D MAP ABOUT X-AXIS.                           *
C  *                                                                       *
C  *    MAY 1991  PURDUE UNIVERSITY VAX 8550 T. BAKER.                     * 
C  *    MODIFIED 11-DEC-1997 BY T. BAKER.                                  *
C  *    LAST MODIFIED 25-FEB-1998 BY T. BAKER (Add UNIX compatibility).    *
C  *************************************************************************
C
      SUBROUTINE MAP_ROTX (MAP1, MAP2, NCOL, NROW, NSEC, RANGL)
C
      IMPLICIT NONE
C
      INTEGER*4 I, J, K, NCOL, NROW, NSEC, IX, IY, TEST
      REAL*4    MAP1(NCOL,NROW,NSEC), MAP2(NROW,NSEC), RANGL
      REAL*4    XCEN, YCEN, A11, A12, A21, A22, PERCENT, DIVISOR
      REAL*4    D1, D2, D3, D4, DX, DY, X, Y
      real*8    deg_to_rad
      LOGICAL   NOTIFY
C  ----------------------------------------------------------------------------
C
      deg_to_rad = acos(-1.0d0)/180.0d0

      TEST = NCOL*NROW*NSEC
      IF(TEST.GT.5000000) THEN
         NOTIFY = .TRUE.
         DIVISOR = FLOAT(NCOL)
         WRITE(*,'(1X, ''  0.0% complete'')')
       ELSE
         NOTIFY = .FALSE.
      ENDIF
C
      XCEN = FLOAT(NROW+1)/2.0
      YCEN = FLOAT(NCOL+1)/2.0
C  ********************************
C  *    SET UP ROTATION MATRIX    *
C  ********************************
      A11 = COS(RANGL*deg_to_rad)
      A12 = SIN(RANGL*deg_to_rad)
      A21 = -A12
      A22 =  A11
C
      DO K = 1,NCOL                        ! STEP THROUGH NCOL YZ SECTIONS
        DO J = 1,NSEC                        ! STEP THROUGH ROWS OF MAP2
          DO I = 1,NROW                  ! STEP THROUGH CURRENT MAP2 ROW 
            X = A11*(FLOAT(I)-XCEN)+A12*(FLOAT(J)-YCEN) + XCEN
            Y = A21*(FLOAT(I)-XCEN)+A22*(FLOAT(J)-YCEN) + YCEN
            IX = X
            IY = Y
            IF(IX.GE.1 .AND. IY.GE.1 .AND. IX.LE.NROW .AND.
     1        IY.LE.NSEC) THEN
               D2        = 0.0
               D3        = 0.0
               D4        = 0.0
               D1        = MAP1(K,IX,IY)
               IF(IX.LT.NROW) D2 = MAP1(K,IX+1,IY)
               IF(IX.LT.NROW.AND.IY.LT.NSEC) D3 = MAP1(K,IX+1,IY+1)
               IF(IY.LT.NSEC) D4 = MAP1(K,IX,IY+1)
               DX        = X - IX
               DY        = Y - IY
               MAP2(I,J) = D1+DX*(D2-D1)+DY*(D4-D1)+DX*DY*(D1-D2+D3-D4)
             ELSE
               MAP2(I,J) = 0.0
            ENDIF
          ENDDO
        ENDDO                                    ! LOOP OVER ROWS
C  **********************************************
C  *    STORE ROTATED SECTION BACK INTO MAP1    *
C  **********************************************
        DO J = 1,NSEC
          DO I = 1,NROW
            MAP1(K,I,J) = MAP2(I,J)
          ENDDO
        ENDDO
        IF(NOTIFY) THEN
          PERCENT = 100.0*(FLOAT(K)/DIVISOR)
          WRITE(*,'(''+'',F5.1)') PERCENT
        ENDIF
      ENDDO                                    ! LOOP OVER SECTIONS
C
      RETURN
      END

C  *************************************************************************
C  *                                                                       *
C  *              *****     SUBROUTINE MAP_ROTX_SLOW    *****              *
C  *                                                                       *
C  *    USE SCRATCH DISK TO ROTATE 3D MAP ABOUT X-AXIS.                    *
C  *                                                                       *
C  *    APRIL 1991  PURDUE UNIVERSITY VAX 8550, T. BAKER.                  * 
C  *    LAST MODIFIED 11-DEC-1997 BY T. BAKER.                             *
C  *************************************************************************
C
      SUBROUTINE MAP_ROTX_SLOW (MAP1, MAP2, TMP, NCOL, NROW, NSEC,
     1                         RANGL)
C
      IMPLICIT NONE
C
      INTEGER*4 I, J, K, NCOL, NROW, NSEC, MOVE_SIZE, TEST
      REAL*4    MAP1(NCOL,NROW,NSEC), MAP2(NCOL,NROW), TMP(NROW)
      REAL*4    RANGL, PERCENT, DIVISOR
      LOGICAL   NOTIFY
C  ----------------------------------------------------------------------------
C
      TEST = NCOL*NROW*NSEC
      IF(TEST.GT.5000000) THEN
         NOTIFY = .TRUE.
       ELSE
         NOTIFY = .FALSE.
      ENDIF
C
      OPEN (UNIT=12, STATUS='SCRATCH', 
     1     ACCESS='SEQUENTIAL', FORM='UNFORMATTED')
      MOVE_SIZE = 4*NROW
C  ***************************************************
C  *    FIRST REARRANGE MAP1 INTO SCRATCH FILE SO    *
C  *     X-AXIS IS THE ROTATION (Z) AXIS             *
C  ***************************************************
      IF(NOTIFY) THEN
        PRINT *, 'Write MAP to scratch file: rotate X-axis to Z
     1 direction'
        WRITE(*,'(1X, ''  0.0% complete'')')
        DIVISOR = FLOAT(NCOL)
      ENDIF
C
      DO J = 1,NCOL                        ! J,K,I <= I,J,K
        DO K = 1,NSEC
          DO I = 1,NROW
            TMP(I) = MAP1(J,I,K)
          ENDDO
          WRITE(12) TMP
        ENDDO
        IF(NOTIFY) THEN
          PERCENT = 100.0*(FLOAT(J)/DIVISOR)
          WRITE(*,'(''+'',F5.1)') PERCENT
        ENDIF
      ENDDO
           REWIND 12
C  **********************************************************
C  *    NOW RELOAD MAP1 WITH DATA STORED IN SCRATCH FILE    *
C  **********************************************************
      IF(NOTIFY) PRINT *, 'Reload MAP from scratch file'
      DO K = 1,NSEC
        DO J = 1,NROW
          READ(12) TMP
          DO I = 1,NCOL
            MAP1(I,J,K) = TMP(I)
CCC            CALL LIB$MOVC3 (MOVE_SIZE, TMP(1), MAP1(1,J,K))
          ENDDO
        ENDDO
      ENDDO
      CLOSE (UNIT=12)
C  **********************************************************
C  *    NOW ROTATE EACH SECTION OF MAP1 BY RANGL DEGREES    *
C  **********************************************************
      IF(NOTIFY) PRINT *, 'Rotate each section by specified angle'
      CALL MAP_DOLLAR_ROT (MAP1, MAP2, NCOL, NROW, NSEC, RANGL)
C
      OPEN (UNIT=12, STATUS='SCRATCH', 
     1     ACCESS='SEQUENTIAL', FORM='UNFORMATTED')
C  **************************************************************
C  *    NOW REWRITE MAP1 INTO SCRATCH FILE TO RESTORE X-AXIS    *
C  **************************************************************
      IF(NOTIFY) THEN
        PRINT *, 'Write MAP to scratch file: restore X-axis'
        WRITE(*,'(1X, ''  0.0% complete'')')
        DIVISOR = FLOAT(NROW)
      ENDIF
C                              ! restore I,J,K <= J,K,I 
      DO J = 1,NROW
        DO I = 1,NCOL
          DO K = 1,NSEC
            TMP(K) = MAP1(I,J,K)
          ENDDO
          WRITE(12) TMP
        ENDDO
        IF(NOTIFY) THEN
          PERCENT = 100.0*(FLOAT(J)/DIVISOR)
          WRITE(*,'(''+'',F5.1)') PERCENT
        ENDIF
      ENDDO
           REWIND 12
C  **********************************************************
C  *    NOW RELOAD MAP1 WITH DATA STORED IN SCRATCH FILE    *
C  **********************************************************
      IF(NOTIFY) PRINT *, 'Reload MAP from scratch file'
      MOVE_SIZE = 4*NSEC
      DO K = 1,NSEC
        DO J = 1,NROW
          READ(12) TMP
          DO I = 1,NCOL
            MAP1(I,J,K) = TMP(I)
CCC            CALL LIB$MOVC3 (MOVE_SIZE, TMP(1), MAP1(1,J,K))
          ENDDO
        ENDDO
      ENDDO
      CLOSE (UNIT=12)
C
      RETURN
      END

C  *************************************************************************
C  *                                                                       *
C  *              *****     SUBROUTINE MAP_ROTY     *****                  *
C  *                                                                       *
C  *    IN CORE ROTATION OF 3D MAP ABOUT Y-AXIS.                           *
C  *                                                                       *
C  *    MAY 1991  PURDUE UNIVERSITY VAX 8550 T. BAKER.                     * 
C  *    LAST MODIFIED 11-DEC-1997 BY T. BAKER.                             *
C  *************************************************************************
C
      SUBROUTINE MAP_ROTY (MAP1, MAP2, NCOL, NROW, NSEC, RANGL)
C
      IMPLICIT NONE
C
      INTEGER*4 I, J, K, NCOL, NROW, NSEC, IX, IY, TEST
      REAL*4    MAP1(NCOL,NROW,NSEC), MAP2(NSEC,NCOL), RANGL
      REAL*4    XCEN, YCEN, A11, A12, A21, A22, PERCENT, DIVISOR
      REAL*4    D1, D2, D3, D4, DX, DY, X, Y
      real*8    deg_to_rad
      LOGICAL   NOTIFY
C  ----------------------------------------------------------------------------
C
      deg_to_rad = acos(-1.0d0)/180.0d0

      TEST = NCOL*NROW*NSEC
      IF(TEST.GT.5000000) THEN
         NOTIFY = .TRUE.
         DIVISOR = FLOAT(NROW)
         WRITE(*,'(1X, ''  0.0% complete'')')
       ELSE
         NOTIFY = .FALSE.
      ENDIF
C
      XCEN = FLOAT(NSEC+1)/2.0
      YCEN = FLOAT(NCOL+1)/2.0
C  ********************************
C  *    SET UP ROTATION MATRIX    *
C  ********************************
      A11 = COS(RANGL*deg_to_rad)
      A12 = SIN(RANGL*deg_to_rad)
      A21 = -A12
      A22 =  A11
C
      DO K = 1,NROW                        ! STEP THROUGH NROW ZX SECTIONS
        DO J = 1,NCOL                        ! STEP THROUGH ROWS OF MAP2
          DO I = 1,NSEC                  ! STEP THROUGH CURRENT MAP2 ROW 
            X = A11*(FLOAT(I)-XCEN)+A12*(FLOAT(J)-YCEN) + XCEN
            Y = A21*(FLOAT(I)-XCEN)+A22*(FLOAT(J)-YCEN) + YCEN
            IX = X
            IY = Y
            IF(IX.GE.1 .AND. IY.GE.1 .AND. IX.LE.NSEC .AND.
     1        IY.LE.NCOL) THEN
               D2        = 0.0
               D3        = 0.0
               D4        = 0.0
               D1        = MAP1(IY,K,IX)
               IF(IX.LT.NSEC) D2 = MAP1(IY,K,IX+1)
               IF(IX.LT.NSEC.AND.IY.LT.NCOL) D3 = MAP1(IY+1,K,IX+1)
               IF(IY.LT.NCOL) D4 = MAP1(IY+1,K,IX)
               DX        = X - IX
               DY        = Y - IY
               MAP2(I,J) = D1+DX*(D2-D1)+DY*(D4-D1)+DX*DY*(D1-D2+D3-D4)
             ELSE
               MAP2(I,J) = 0.0
            ENDIF
          ENDDO
        ENDDO                                    ! LOOP OVER ROWS
C  **********************************************
C  *    STORE ROTATED SECTION BACK INTO MAP1    *
C  **********************************************
        DO J = 1,NCOL
          DO I = 1,NSEC
            MAP1(J,K,I) = MAP2(I,J)
          ENDDO
        ENDDO
        IF(NOTIFY) THEN
          PERCENT = 100.0*(FLOAT(K)/DIVISOR)
          WRITE(*,'(''+'',F5.1)') PERCENT
        ENDIF
      ENDDO                                    ! LOOP OVER SECTIONS
C
      RETURN
      END

C  *************************************************************************
C  *                                                                       *
C  *              *****     SUBROUTINE MAP_ROTY_SLOW    *****              *
C  *                                                                       *
C  *    USE SCRATCH DISK TO ROTATE 3D MAP ABOUT Y-AXIS.                    *
C  *                                                                       *
C  *    APRIL 1991  PURDUE UNIVERSITY VAX 8550, T. BAKER.                  *
C  *    LAST MODIFIED 26-FEB-1998 BY T. BAKER (Make UNIX compatible).      *
C  *************************************************************************
C
      SUBROUTINE MAP_ROTY_SLOW (MAP1, MAP2, TMP, NCOL, NROW, NSEC,
     1                         RANGL)
C
      IMPLICIT NONE
C
      INTEGER*4 I, J, K, NCOL, NROW, NSEC, MOVE_SIZE, TEST
      REAL*4    MAP1(NCOL,NROW,NSEC), MAP2(NCOL,NROW), TMP(NSEC)
      REAL*4    RANGL, PERCENT, DIVISOR
      LOGICAL   NOTIFY
C  ----------------------------------------------------------------------------
C
      TEST = NCOL*NROW*NSEC
      IF(TEST.GT.5000000) THEN
         NOTIFY = .TRUE.
       ELSE
         NOTIFY = .FALSE.
      ENDIF
C
      OPEN (UNIT=12, STATUS='SCRATCH', 
     1     ACCESS='SEQUENTIAL', FORM='UNFORMATTED')
      MOVE_SIZE = 4*NSEC
C  ***************************************************
C  *    FIRST REARRANGE MAP1 INTO SCRATCH FILE SO    *
C  *     Y-AXIS IS THE ROTATION (Z) AXIS             *
C  ***************************************************
      IF(NOTIFY) THEN
        PRINT *, 'Write MAP to scratch file: rotate Y-axis to Z
     1 direction'
        WRITE(*,'(1X, ''  0.0% complete'')')
        DIVISOR = FLOAT(NROW)
      ENDIF
C
      DO J = 1,NROW                        ! K,I,J <= I,J,K
        DO I = 1,NCOL
          DO K = 1,NSEC
            TMP(K) = MAP1(I,J,K)
          ENDDO
          WRITE(12) TMP
        ENDDO
        IF(NOTIFY) THEN
          PERCENT = 100.0*(FLOAT(J)/DIVISOR)
          WRITE(*,'(''+'',F5.1)') PERCENT
        ENDIF
      ENDDO
           REWIND 12
C  **********************************************************
C  *    NOW RELOAD MAP1 WITH DATA STORED IN SCRATCH FILE    *
C  **********************************************************
      IF(NOTIFY) PRINT *, 'Reload MAP from scratch file'
      DO J = 1,NROW
        DO I = 1,NCOL
          READ(12) TMP
          DO K = 1,NSEC
            MAP1(K,I,J) = TMP(K)
CCC            CALL LIB$MOVC3 (MOVE_SIZE, TMP(1), MAP1(1,I,J))
          ENDDO
        ENDDO
      ENDDO
      CLOSE (UNIT=12)
C  **********************************************************
C  *    NOW ROTATE EACH SECTION OF MAP1 BY RANGL DEGREES    *
C  **********************************************************
      IF(NOTIFY) PRINT *, 'Rotate each section by specified angle'
      CALL MAP_DOLLAR_ROT (MAP1, MAP2, NCOL, NROW, NSEC, RANGL)
C
      OPEN (UNIT=12, STATUS='SCRATCH', 
     1     ACCESS='SEQUENTIAL', FORM='UNFORMATTED')
C  **************************************************************
C  *    NOW REWRITE MAP1 INTO SCRATCH FILE TO RESTORE Y-AXIS    *
C  **************************************************************
      IF(NOTIFY) THEN
        PRINT *, 'Write MAP to scratch file: restore Y-axis'
        WRITE(*,'(1X, ''  0.0% complete'')')
        DIVISOR = FLOAT(NSEC)
      ENDIF
C                              ! restore I,J,K <= K,I,J
      DO K = 1,NSEC
        DO J = 1,NROW
          DO I = 1,NCOL
            TMP(I) = MAP1(K,I,J)
          ENDDO
          WRITE(12) TMP
        ENDDO
        IF(NOTIFY) THEN
          PERCENT = 100.0*(FLOAT(K)/DIVISOR)
          WRITE(*,'(''+'',F5.1)') PERCENT
        ENDIF
      ENDDO
           REWIND 12
C  **********************************************************
C  *    NOW RELOAD MAP1 WITH DATA STORED IN SCRATCH FILE    *
C  **********************************************************
      IF(NOTIFY) PRINT *, 'Reload MAP from scratch file'
      MOVE_SIZE = 4*NCOL
      DO K = 1,NSEC
        DO J = 1,NROW
          READ(12) TMP
          DO I = 1,NCOL
CCC            CALL LIB$MOVC3 (MOVE_SIZE, TMP(1), MAP1(1,J,K))
            MAP1(I,J,K) = TMP(I)
          ENDDO
        ENDDO
      ENDDO
      CLOSE (UNIT=12)
C
      RETURN
      END

C  **************************************************************************
C  *                                                                        *
C  *               *****     SUBROUTINE MAP_SCALE     *****                 *
C  *                                                                        *
C  *    ROUTINE TO RESCALE "MAP" VALUES INSIDE THE RANGE MIN_OLD-MAX_OLD    *
C  *     TO MIN_NEW-MAX_NEW.                                                *
C  *                                                                        *
C  *    WORKS WITH 2D OR 3D "MAP" DATA.                                     *
C  *                                                                        *
C  *    JANUARY 1985  PURDUE UNIVERSITY VAX 11/750, T. BAKER.               *
C  *      Modified screen output, 22 Sept 1994, by D. Belnap                *
C  *    MODIFIED 16-DEC-1997 BY T. BAKER (Rid of EM.CMM).                   *
C  *    LAST MODIFIED 27-FEB-1998 BY T. BAKER (Make UNIX compatible).       *
C  **************************************************************************
C
      SUBROUTINE MAP_SCALE (LUN, MAP, NCOL, NROW, NSEC, MIN_OLD,
     1 MAX_OLD, MIN_NEW, MAX_NEW, MAP_MIN, MAP_MAX, 
     2 MAP_AVG, MAP_STD, MAP_SCALEF, FILE_ID, MAP_INCORE)
C
      IMPLICIT NONE
C
      include '../include/vax_minmax.inc'
C
      INTEGER*4 NCOL, NROW, NSEC, LUN, I, J, K, ARRAY_SIZE, FILE_ID
      REAL*4    MAP(1), MIN_OLD, MAX_OLD, MIN_NEW, MAX_NEW, DIV, KSTEP
      REAL*4    MAP_MIN, MAP_MAX, MAP_AVG, MAP_STD, MAP_SCALEF
      REAL*4    SCALE, TMP2, SUMSQ, TOT, VAR, PERCENT
      LOGICAL   NOTIFY, MAP_INCORE

      integer alloc_stat
      real*4, allocatable :: tmp(:)
      integer*2, allocatable :: itmp(:)
C  ----------------------------------------------------------------------------
C
      allocate(itmp(ncol), stat=alloc_stat)
      allocate(tmp(ncol), stat=alloc_stat)

      IF(.NOT.MAP_INCORE) THEN
        PRINT *, '*************************************************'
        PRINT *, '* MAP not in core memory - code can not be used *'
        PRINT *, '*************************************************'
        RETURN
      ENDIF
      ARRAY_SIZE = NCOL*NROW*NSEC
      DIV        = FLOAT(ARRAY_SIZE)
C
      IF(NSEC.GT.1) THEN
         PRINT *,'*****************************'
         PRINT *,'*  Rescaling 3D MAP values  *'
         PRINT *,'*****************************'
         PRINT *,'OLD values'
         PRINT *,'MIN_OLD,MAX_OLD =',MIN_OLD, MAX_OLD      
         PRINT *,'NEW values'
         PRINT *,'MIN_NEW,MAX_NEW =',MIN_NEW, MAX_NEW
         PERCENT = 0.0
         WRITE(*,'(1X,F5.1,''% done with 3D MAP'')') PERCENT
         NOTIFY = .TRUE.
       ELSE
         NOTIFY = .FALSE.
      ENDIF
C
      SCALE = 1.0
      IF(MIN_OLD.NE.MAX_OLD) THEN
        SCALE = (MAX_NEW-MIN_NEW)/(MAX_OLD-MIN_OLD)
      ENDIF
      MAP_MIN = R4_MAX
      MAP_MAX = R4_MIN
      SUMSQ   = 0.0
      TOT     = 0.0
C
      KSTEP = DIV/10.0
      DO I = 1,ARRAY_SIZE
        TMP2 = MAP(I)
        IF(TMP2.GE.MIN_OLD.AND.TMP2.LE.MAX_OLD) THEN
          TMP2   = MIN_NEW + (TMP2-MIN_OLD) * SCALE
          MAP(I) = TMP2
        ENDIF
        SUMSQ   = SUMSQ + TMP2*TMP2
        TOT     = TOT + TMP2
        MAP_MIN = AMIN1(MAP_MIN,TMP2)
        MAP_MAX = AMAX1(MAP_MAX,TMP2)
        IF(FLOAT(I).GE.KSTEP) THEN
          IF(NOTIFY) THEN
            PERCENT = 100.0*FLOAT(I)/DIV
            WRITE(*,'(''+'',F5.1)') PERCENT
            KSTEP = KSTEP + DIV/10.0
          ENDIF
        ENDIF
      ENDDO
C
      MAP_AVG = TOT/DIV
      MAP_STD = 0.0
      VAR     = (SUMSQ/DIV) - (MAP_AVG*MAP_AVG)
      IF(VAR.GE.0.0) MAP_STD = SQRT(VAR)
C
      deallocate(tmp, itmp)
      RETURN
      END


C  ****************************************************************************
C  *                                                                          *
C  *                  *****     SUBROUTINE MAP_SHIFT     *****                *
C  *                                                                          *
C  *    SHIFT DATA WITHIN 2D MAP BY XSHIFT,YSHIFT.                            *
C  *                                                                          *
C  *    POINT LAYOUT FOR BI-LINEAR INTERPOLATION SCHEME :                     *
C  *                                                                          *
C  *          D4            D3                                                *
C  *                                                                          *
C  *                    + (dx,dy)                                             *
C  *                    .                                                     *
C  *          D1. . . . .   D2                                                *
C  *                                                             *
C  *    THE INTERPOLATION ALGORITHM IS COMPUTED AS FOLLOWS:                   *
C  *                                                                          *
C  *          D1(1-dX)(1-dY) + D2(dX)(1-dY) + D3(dX)(dY) + D4(1-dX)(dY)       *
C  *                                                                          *
C  *    WHICH REDUCES TO:                                                     *
C  *                                                                          *
C  *          D1 + (D2-D1)dx + (D4-D1)dy + (D1-D2+D3-D4)dxdy                  *
C  *                                                                          *
C  *    MAY 1989 PURDUE UNIVERSITY VAX 8550, T. BAKER                         *
C  *    LAST MODIFIED 16-MAY-89 BY T. BAKER                                   *
C  ****************************************************************************
C
      SUBROUTINE MAP_SHIFT (MAP, NCOL, NROW, XSHIFT, YSHIFT)
C
      IMPLICIT NONE
C
      INTEGER*4 I, J, IX1, IX2, IY1, IY2, NCOL, NROW
      REAL*4    MAP(NCOL,NROW)
      REAL*4    X, Y, DX, DY, XSHIFT, YSHIFT, D1, D2, D3, D4
      LOGICAL   XSTART, XEND, YSTART, YEND

      integer alloc_stat
      real*4, allocatable :: map2(:,:)
C  ----------------------------------------------------------------------------
C
      allocate(map2(ncol,nrow), stat=alloc_stat)
      if(alloc_stat > 0) then
         write(*,*) 'Memory allocation failed in map_shift'
      endif

      DO J = 1,NROW
        Y      = J - YSHIFT
        IY1    = INT(Y)
        IY2    = IY1 + 1
        DY     = Y - IY1
        YSTART = .TRUE.
        YEND   = .TRUE.
        IF(IY1.LT.1 .OR. IY1.GT.NROW) YSTART = .FALSE.
        IF(IY2.LT.1 .OR. IY2.GT.NROW) YEND   = .FALSE.
        DO I = 1,NCOL
          X      = I - XSHIFT
          IX1    = INT(X)
          IX2    = IX1 + 1
          DX     = X - IX1
          D1     = 0.0
          D2     = 0.0
          D3     = 0.0
          D4     = 0.0
          XSTART = .TRUE.
          XEND   = .TRUE.
          IF(IX1.LT.1 .OR. IX1.GT.NCOL) XSTART = .FALSE.
          IF(IX2.LT.1 .OR. IX2.GT.NCOL) XEND   = .FALSE.
          IF(XSTART.AND.YSTART) D1 = MAP(IX1,IY1)
          IF(XEND.AND.YSTART)   D2 = MAP(IX2,IY1)
          IF(XEND.AND.YEND)     D3 = MAP(IX2,IY2)
          IF(XSTART.AND.YEND)   D4 = MAP(IX1,IY2)
          MAP2(I,J) = D1 + (D2-D1)*DX + (D4-D1)*DY
     1                  + (D1-D2+D3-D4)*DX*DY
        ENDDO
      ENDDO
C
      CALL COPY_R4 (MAP2, MAP, NCOL*NROW)            ! COPY "MAP2" INTO "MAP"
C
      deallocate(map2)
      RETURN
      END

C  **********************************************************************
C  *                                                                    *
C  *              *****     SUBROUTINE MAP_STATS     *****              *
C  *                                                                    *
C  *    ROUTINE TO DETERMINE THE MIN/MAX, AVERAGE AND STD FOR VALUES    *
C  *     IN THE "MAP" ARRAY.                                            *
C  *                                                                    *
C  *    APRIL 1984  PURDUE UNIVERSITY VAX 11/750, T. BAKER.             *
C  *    LAST MODIFIED 17-DEC-1997 BY T. BAKER (RID OF EM.CMM).          *
C  **********************************************************************
      SUBROUTINE MAP_STATS (LUN, MAP, NCOL, NROW, NSEC, MAP_MIN,
     1 MAP_MAX, MAP_AVG, MAP_STD, EM_WRT)
C
      IMPLICIT NONE
C
      include '../include/vax_minmax.inc'
C
      INTEGER*4 LUN, NCOL, NROW, NSEC, I, J, K, ARRAY_SIZE, KSTEP
      INTEGER*4 EM_WRT
      REAL*4    MAP(1), MAP_MIN, MAP_MAX, MAP_AVG, MAP_STD
      REAL*4    VAR, X, STD2, VAR2, DIV, PERCENT
      REAL*8    TOT, SUMSQ
      LOGICAL   NOTIFY

      integer alloc_stat
      real*4, allocatable :: tmp(:)
C  ----------------------------------------------------------------------------
C
      allocate(tmp(ncol), stat=alloc_stat)
      
      ARRAY_SIZE = NCOL*NROW*NSEC
C
      IF(EM_WRT.EQ.1) WRITE(*,1000) 
      IF(EM_WRT.EQ. 1. AND. ARRAY_SIZE.GT.5000000) THEN
         PERCENT = 0.0
         WRITE(*,'(''+'',F5.1,''% done with 3D MAP'')') PERCENT
         NOTIFY = .TRUE.
       ELSE
         NOTIFY = .FALSE.
      ENDIF
C
      MAP_MIN  = R4_MAX
      MAP_MAX  = R4_MIN
      SUMSQ    = 0.0
      TOT      = 0.0
C
      KSTEP = NINT(FLOAT(ARRAY_SIZE)/10.0)
      DIV   = FLOAT(ARRAY_SIZE)
      DO I = 1,ARRAY_SIZE
        X       = MAP(I)
        SUMSQ   = SUMSQ + X**2
        TOT     = TOT + X
        MAP_MIN = AMIN1(MAP_MIN,X)
        MAP_MAX = AMAX1(MAP_MAX,X)
        IF(MOD(I,KSTEP).EQ.0) THEN
          IF(NOTIFY) THEN
            PERCENT = 100.0*FLOAT(I)/DIV
            WRITE(*,'(''+'',F5.1)') PERCENT
          ENDIF
        ENDIF
      ENDDO
C
Ca      print *, 'mapstats tot == ', tot
Ca      print *, 'array size == ', array_size
      MAP_AVG = TOT/FLOAT(ARRAY_SIZE)
      MAP_STD = 0.0
      VAR     = (SUMSQ/FLOAT(ARRAY_SIZE))-(MAP_AVG*MAP_AVG)
      IF(VAR.GE.0.0) MAP_STD = SQRT(VAR)
      IF(EM_WRT.EQ.1) WRITE(*,1010) MAP_MIN, MAP_MAX, MAP_AVG, MAP_STD
C
      deallocate(tmp)
      RETURN
 1000 FORMAT('      MINIMUM     MAXIMUM        AVERAGE         STD',
     1     /,' -----------------------------------------------------',/)
 1010      FORMAT('+',2F12.1,2F15.2)
      END

C  ****************************************************************************
C  *                                                                          *
C  *                  *****     SUBROUTINE MAP_SYM     *****                  *
C  *                                                                          *
C  *    APPLY NROT ROTATIONAL SYMMETRY TO MAP AT XCEN, YCEN                   *
C  *                                                                          *
C  *    MAP(2) ARE THE VIRTUAL ARRAYS CONTAINING THE ORIGINAL AND SYMMETRIZED *
C  *     MAP DATA RESPECTIVELY.                                               *
C  *                                                                          *
C  *    LIMIT ON NROT = 100, IF NROT>100 THEN COMPUTE CIRCULAR AVERAGE        *
C  *                                                                          *
C  *    MAY 1992  PURDUE UNIVERSITY VAX 8550, T. BAKER.                       *
C  *    LAST MODIFIED 17-DEC-1997 BY T. BAKER (RID OF EM.CMM).                *
C  ****************************************************************************
C
      SUBROUTINE MAP_SYM (MAP, NCOL, NROW, XCEN, YCEN, NROT, MAP_MIN,
     1 MAP_MAX, MAP_AVG, MAP_STD, EM_WRT)
C
      IMPLICIT NONE
C
      INTEGER*4 NROT, NCOL, NROW, I, ARRAY_SIZE, EM_WRT
      REAL*4    XCEN, YCEN, MAP(1), MAP_MIN, MAP_MAX, MAP_AVG, MAP_STD
      REAL*4, allocatable :: MAP2(:)
      integer   alloc_stat
C  ----------------------------------------------------------------------------
C
      allocate(map2(ncol*nrow), stat=alloc_stat)
      if(alloc_stat > 0) then
         write(*,*) 'Unable to allocate MAP2 in MAP_SYM'
         call tsbend
      endif

      IF(NROT.GT.100) THEN
         CALL MAP_SYM_CAVG (MAP, MAP2, NCOL, NROW, XCEN, YCEN, 
     1   NROT, MAP_MIN, MAP_MAX, MAP_AVG, MAP_STD, EM_WRT)
       ELSE
         CALL MAP_SYM_RAVG (MAP, MAP2, NCOL, NROW, XCEN, YCEN, 
     1   NROT, MAP_MIN, MAP_MAX, MAP_AVG, MAP_STD, EM_WRT)
      ENDIF
C
      ARRAY_SIZE = NCOL*NROW
      CALL COPY_R4 (MAP2, MAP, ARRAY_SIZE)

      deallocate(map2)

      CALL MAP_STATS (1, MAP, NCOL, NROW, 1, MAP_MIN, MAP_MAX, 
     1 MAP_AVG, MAP_STD, 0)
C
      RETURN
      END

C  ****************************************************************
C  *                                                              *
C  *          *****     SUBROUTINE MAP_SYM_CAVG     *****         *
C  *                                                              *
C  *    NROT > 100: COMPUTE CIRCULARLY AVERAGED MAP SECTION.      *
C  *                                                              *
C  *    MAY 1992  PURDUE UNIVERSITY VAX 8550, T. BAKER.           *
C  *    LAST MODIFIED 17-DEC-1997 BY T. BAKER (RID OF EM.CMM).    *
C  ****************************************************************
C
      SUBROUTINE MAP_SYM_CAVG (MAP1, MAP2, NCOL, NROW, XCEN, YCEN,
     1 NROT, MAP_MIN, MAP_MAX, MAP_AVG, MAP_STD, EM_WRT)
C
      IMPLICIT NONE
C
      INTEGER*4 NROT, NCOL, NROW, EM_WRT
Ca      INTEGER*2 INUM, IR, I, J, K, INTERP, IRNUM(10000)
      INTEGER*4 INUM, IR, I, J, K, INTERP, IRNUM(10000)
      REAL*4    MAP1(NCOL,NROW), MAP2(NCOL,NROW), MAP_MIN, MAP_MAX
      REAL*4    MAP_AVG, MAP_STD, XCEN, YCEN, X, Y, YSQ, RINT(10000)
      PARAMETER (INTERP=3)
C  ----------------------------------------------------------------------------
C
      INUM = NINT(INTERP*SQRT(FLOAT(NROW)**2+FLOAT(NCOL)**2))
      if(inum.gt.10000) print *, 'Problem in MAY_SYM_CAVG'
C
      DO I = 1,INUM            ! INITIALIZE 1D INTERPOLATION ARRAY
        RINT(I)  = 0.0
        IRNUM(I) =   0
      ENDDO
C  *************************************************************
C  *    STEP THROUGH IMAGE AND REDUCE TO 1D CIRCULAR AVERAGE   *
C  *************************************************************
      DO J = 1,NROW
        Y   = FLOAT(J)-YCEN
        YSQ = Y*Y
        DO I = 1,NCOL
          X         = FLOAT(I)-XCEN
          IR        = NINT(INTERP*SQRT(X*X+YSQ))+1
          IRNUM(IR) = IRNUM(IR)+1
          RINT(IR)  = RINT(IR) + MAP1(I,J)
        ENDDO
      ENDDO
C  ********************************
C  *    NORMALIZE THE 1D ARRAY    *
C  ********************************
      DO I = 1,INUM
        IF(IRNUM(I).GT.1) RINT(I) = RINT(I)/FLOAT(IRNUM(I))
      ENDDO
C  **************************************************
C  *    FILL MAP2 WITH CIRCULARLY AVERAGED IMAGE    *
C  **************************************************
      DO J = 1,NROW
        Y   = FLOAT(J)-YCEN
        YSQ = Y*Y
        DO I = 1,NCOL
          X         = FLOAT(I)-XCEN
          IR        = NINT(INTERP*SQRT(X*X+YSQ))+1
          MAP2(I,J) = NINT(RINT(IR))
        ENDDO
      ENDDO
C
      CALL MAP_STATS (1, MAP2, NCOL, NROW, 1, MAP_MIN, MAP_MAX, 
     1 MAP_AVG, MAP_STD, EM_WRT)
C
      RETURN
      END

C  *************************************************************
C  *                                                           *
C  *        *****     SUBROUTINE MAP_SYM_GRID     *****        *
C  *                                                           *
C  *    FIND POSITIONS OF SYMMETRY RELATED POINTS.             *
C  *    LIMIT: NROT<101                                        *
C  *                                                           *
C  *    MAY 1992  PURDUE UNIVERSITY VAX 8550, T. BAKER.        *
C  *    LAST MODIFIED 25-MAY-1992 BY T. BAKER.                 *
C  *************************************************************
C
      SUBROUTINE MAP_SYM_GRID (I,J,NROT,XCEN,YCEN,XGRID,YGRID)
C
      IMPLICIT NONE
C
      COMMON /ROT_MATRIX/ A11, A12, A21, A22
      REAL*4    XCEN, YCEN, XGRID(100), YGRID(100)
      REAL*4    A11, A12, A21, A22
      INTEGER*4 I, J, K, NROT
C  ----------------------------------------------------------------------------
C
      XGRID(1) = J                              ! CURRENT COLUMN POSITION
      YGRID(1) = I                              ! CURRENT ROW POSITION
      DO K = 2,NROT
        XGRID(K) = A11*(XGRID(K-1)-XCEN)+A12*(YGRID(K-1)-YCEN)+XCEN
        YGRID(K) = A21*(XGRID(K-1)-XCEN)+A22*(YGRID(K-1)-YCEN)+YCEN
      ENDDO
C
      RETURN
      END

C  ****************************************************************
C  *                                                              *
C  *        *****     SUBROUTINE MAP_SYM_RAVG     *****           *
C  *                                                              *
C  *    NROT < 101: COMPUTE ROTATIONAL AVERAGE OF IMAGE           *
C  *                                                              *
C  *    MAY 1992  PURDUE UNIVERSITY VAX 8550, T. BAKER.           *
C  *    LAST MODIFIED 17-DEC-1997 BY T. BAKER (RID OF EM.CMM).    *
C  ****************************************************************
C
      SUBROUTINE MAP_SYM_RAVG (MAP1, MAP2, NCOL, NROW, XCEN, YCEN, 
     1 NROT, MAP_MIN, MAP_MAX, MAP_AVG, MAP_STD, EM_WRT)
C
      IMPLICIT NONE
C
      COMMON /ROT_MATRIX/ A11, A12, A21, A22
      INTEGER*4 NROT, NCOL, NROW, I, J, K, EM_WRT
      INTEGER*4 IX, IY, INUM, NUM, IR
      INTEGER*4 NGRID, ITST, ILST
      REAL*4    MAP1(NCOL,NROW), MAP2(NCOL,NROW), OUTSIDE
      REAL*4    MAP_MIN, MAP_MAX, MAP_AVG, MAP_STD
      REAL*4    XCEN, YCEN, A11, A12, A21, A22, DENS, D1, D2, D3, D4
      REAL*4    DX, DY, TOT, XGRID(100), YGRID(100), X, Y
      real*8    deg_to_rad
      DATA OUTSIDE/-9999.9/
C  ----------------------------------------------------------------------------

      deg_to_rad = acos(-1.0d0)/180.0d0
C
C  ********************************
C  *    SET UP ROTATION MATRIX    *
C  ********************************
      A11 = COS(deg_to_rad*360.0/FLOAT(NROT))
      A12 = SIN(deg_to_rad*360.0/FLOAT(NROT))
      A21 = -A12
      A22 =  A11
C
      DO J = 1,NCOL                  ! STEP THROUGH COLUMNS OF MAP1
        DO I = 1,NROW                  ! STEP THROUGH CURRENT COLUMN OF MAP1
          CALL MAP_SYM_GRID (I, J, NROT, XCEN, YCEN, XGRID, YGRID)
C           **********************************************************
C           *   CHECK NUMBER OF GRID POINTS WITHIN IMAGE BOUNDARY    *
C           **********************************************************
          NGRID = 1
          DO K = 2,NROT
            IF(XGRID(K).GE.1.0.AND.XGRID(K).LE.NCOL.AND.
     1        YGRID(K).GE.1.0.AND.YGRID(K).LE.NROW) THEN
               NGRID = NGRID+1
             ELSE
               XGRID(K) = -1.0
               YGRID(K) = -1.0
            ENDIF
          ENDDO
          IF(NGRID.EQ.1) THEN
             MAP2(J,I) = OUTSIDE
           ELSE
             DENS  = MAP1(J,I)
             NGRID = 1
             DO K = 2,NROT
               IX = XGRID(K)
               IY = YGRID(K)
               IF(IX.GE.1.AND.IY.GE.1) THEN
                 NGRID = NGRID+1
                 D2   = 0.0
                 D3   = 0.0
                 D4   = 0.0
                 D1   = MAP1(IX,IY)
                 IF(IX.LT.NCOL) D2 = MAP1(IX+1,IY)
                 IF(IY.LT.NROW.AND.IX.LT.NCOL) D3 = MAP1(IX+1,IY+1)
                 IF(IY.LT.NROW) D4 = MAP1(IX,IY+1)
                 DX   = XGRID(K)-IX
                 DY   = YGRID(K)-IY
                 DENS = DENS+D1+DX*(D2-D1)+DY*(D4-D1)+
     1                      DX*DY*(D1-D2+D3-D4)
               ENDIF
               ENDDO
             MAP2(J,I) = DENS/FLOAT(NGRID)
          ENDIF
        ENDDO
      ENDDO
C  *********************************************************
C  *    COMPUTE BACKGROUND VALUE FROM POINTS ON BOUNDARY   *
C  *********************************************************
      TOT  = 0.0
      NUM  =   0
      DO J = 1,NROW                              ! STEP THROUGH ROW BY ROW
        ITST = -1                                   ! OUTSIDE BOUNDARY
        ILST =  0
        DO I = 1,NCOL                                 ! STEP THROUGH ROW J
          IF(MAP2(I,J).EQ.OUTSIDE) THEN                 ! OUTSIDE BOUNDARY
             IF(ITST.EQ.1.AND.I.NE.ILST) THEN            ! JUST LEAVING SYM AREA
               NUM = NUM+1
               TOT = TOT + MAP1(I,J)                  ! PIXEL AT RIGHT BOUNDARY
               ITST = -1                         ! NOW OUTSIDE BOUNDARY
               GO TO 100                         ! STEP TO NEXT ROW (J+1)
             ENDIF
             ITST = -1                             ! OUTSIDE BOUNDARY
           ELSE                                    ! INSIDE BOUNDARY
             IF(ITST.EQ.-1.AND.I.GT.1) THEN           ! LEAVING OUTSIDE/ENTERING
               NUM  = NUM+1                       !         INSIDE
               ILST = I+1
               TOT  = TOT + MAP1(I-1,J)  ! PREVIOUS PIXEL WAS ON LEFT BOUNDARY
             ENDIF
             ITST = 1                                    ! INSIDE BOUNDARY
          ENDIF
        ENDDO
        IF(ILST.EQ.0) THEN
C           **********************************************
C           *    GET HERE ONLY WHEN ENTIRE ROW IS ALL    *
C           *     INSIDE OR ALL OUTSIDE THE BOUNDARY     *
C           **********************************************
          IF(ITST.EQ.1) THEN                     ! ENTIRE ROW INSIDE BOUNDARY
            IF(NUM.EQ.0) THEN                   ! FIRST ROW INSIDE BOUNDARY USES
               DO I = 1,NCOL                   !      ENTIRE ROW FOR BACKGROUND
                 NUM = NUM + 1
                 TOT = TOT + MAP1(I,J)
               ENDDO
             ELSE                         ! USE EDGE VALUES FOR BACKGROUND
               NUM = NUM + 2            
               TOT = TOT + MAP1(1,J) + MAP1(NCOL,J)
            ENDIF
          ENDIF
          IF(ITST.EQ.-1) THEN                    ! ENTIRE ROW OUTSIDE BOUNDARY
            IF(NUM.NE.0) THEN                             ! THIS IS LAST ROW
              DO I = 1,NCOL
                NUM = NUM + 1
                TOT = TOT + MAP1(I,J-1)
              ENDDO
            ENDIF
          ENDIF
        ENDIF
  100      ENDDO
CCC      IMG_BKG = NINT(TOT/FLOAT(NUM))
C  ***********************************************
C  *    REPLACE ALL VALUES = OUTSIDE WITH 0.0    *
C  ***********************************************
      DO J = 1,NROW
        DO I = 1,NCOL
          IF(MAP2(I,J).EQ.OUTSIDE) MAP2(I,J) = 0.0
        ENDDO
      ENDDO
C
      CALL MAP_STATS (1, MAP2, NCOL, NROW, 1, MAP_MIN, MAP_MAX, 
     1 MAP_AVG, MAP_STD, EM_WRT)
C
      RETURN
      END

C  **************************************************************************
C  *                                                                        *
C  *                  *****     SUBROUTINE MAP_TRP     *****                *
C  *                                                                        *
C  *    INTERPOLATES AN NCOL BY NROW MAP PRODUCING AN NCOL2 BY NROW2 MAP    *
C  *                                                                        *
C  *    POINT LAYOUT FOR BI-LINEAR INTERPOLATION SCHEME :                   *
C  *                                                                        *
C  *          D4            D3                                              *
C  *                                                                        *
C  *                    + (dx,dy)                                           *
C  *                    .                                                   *
C  *          D1. . . . .   D2                                              *
C  *                                                           *
C  *    THE INTERPOLATION ALGORITHM IS COMPUTED AS FOLLOWS:                 *
C  *                                                                        *
C  *          D1(1-dX)(1-dY) + D2(dX)(1-dY) + D3(dX)(dY) + D4(1-dX)(dY)     *
C  *                                                                        *
C  *    WHICH REDUCES TO:                                                   *
C  *                                                                        *
C  *          D1 + (D2-D1)dx + (D4-D1)dy + (D1-D2+D3-D4)dxdy                *
C  *                                                           *
C  *    JULY 1987  PURDUE UNIVERSITY, VAX 11/750, TIM BAKER.                *
C  *    LAST MODIFIED 12-DEC-1997 BY T. BAKER (RID OF EM.CMM).              *
C  **************************************************************************
C
      SUBROUTINE MAP_TRP (MAP1, MAP2, NCOL, NROW, NCOL2, NROW2)
C
      IMPLICIT NONE
C
      INTEGER*4 INDX1, INDX2, INDY1, INDY2, IX10, I, J, IY, IX1
      INTEGER*4 NCOL, NROW, NCOL2, NROW2
      REAL*4    MAP1(NCOL,NROW), MAP2(NCOL2,NROW2)
      REAL*4    XSTEP, YSTEP, Y, X1, D1, D2, D3, D4, DX, DY
C  ----------------------------------------------------------------------------
C
      IX10  = 10*NCOL
      XSTEP = (FLOAT(NCOL)-1.0)/(FLOAT(NCOL2)-1.0)
      YSTEP = (FLOAT(NROW)-1.0)/(FLOAT(NROW2)-1.0)
C
      DO J = 1,NROW2
         Y    = (J-1)*YSTEP
        IY    = Y
        DY    = Y-IY
        INDY1 = IY+1
        INDY2 = 1+MOD(INDY1,NROW)
        DO I = 1,NCOL2
          X1        = (I-1)*XSTEP
          IX1       = X1+IX10
          INDX1     = 1+MOD(IX1,NCOL)
          INDX2     = 1+MOD(INDX1,NCOL)
          D1        = MAP1(INDX1,INDY1)
          D2        = MAP1(INDX2,INDY1)
          D3        = MAP1(INDX2,INDY2)
          D4        = MAP1(INDX1,INDY2)
          IX1       = IX1-IX10
          DX        = X1-IX1
          MAP2(I,J) = D1+DX*(D2-D1)+DY*(D4-D1)+DX*DY*(D1-D2+D3-D4)
        ENDDO
      ENDDO
C
       RETURN
       END

C  ********************************************************************
C  *                                                                  *
C  *              *****     SUBROUTINE MAP_VOL     *****              *
C  *                                                                  *
C  *    DETERMINE THE AREA/VOLUME ABOVE A CUTOFF VALUE                *
C  *                                                                  *
C  *    LUN1 = LOGICAL UNIT CONTAINING MAP FILE                       *
C  *                                                                  *
C  *    JULY 1987  PURDUE UNIVERSIRTY VAX 11/750, L. MATTERN.         *
C  *    LAST MODIFIED 27-FEB-1998 BY T. BAKER (Make UNIX compatible). *
C  ********************************************************************
C
      SUBROUTINE MAP_VOL (LUN, MAP, NCOL, NROW, NSEC, TMP, MAP_MIN, 
     1 MAP_MAX, MAP_INCORE)
C
      IMPLICIT NONE
C
      INTEGER*4 NCOL, NROW, NSEC, NBINS, NBINS_LIM, I, J, K, N, LUN
      PARAMETER (NBINS_LIM=1000)
      INTEGER*4 BIN(NBINS_LIM)
      REAL*4    MAP(NCOL,NROW,NSEC), TMP(NCOL), TOTAL, SIZE
      REAL*4    DIFF, CUTOFF, FRACTION(NBINS_LIM), MAP_MIN, MAP_MAX
      LOGICAL   MAP_INCORE
C -----------------------------------------------------------------------------
C
      IF(.NOT.MAP_INCORE) THEN
        PRINT *, '*************************************************'
        PRINT *, '* MAP not in core memory - code can not be used *'
        PRINT *, '*************************************************'
        RETURN
      ENDIF
  100      WRITE(*,'(''$# OF LEVELS TO SUBDIVIDE THE MAP DENSITIES '',
     1 ''INTO (<CR> = 100): '')')
      READ (*,*,ERR=100) NBINS
      IF(NBINS.LT.0.OR.NBINS.GT.NBINS_LIM.OR.NBINS.EQ.1) THEN
         PRINT *, '******************************'
         PRINT *, '* That number is not allowed *'
         PRINT *, '*    The limits are 2-1000   *'
         PRINT *, '******************************'
         GO TO 100
       ELSE IF(NBINS.EQ.0) THEN
         NBINS = 100
      ENDIF
      DO I = 1,NBINS                  ! INITIALIZE BIN AND FRACTION ARRAYS
        BIN(I)      =   0
        FRACTION(I) = 0.0
      ENDDO
      SIZE = FLOAT(NBINS - 1)
C
      IF(NSEC.GT.1) PRINT *,'     % FINISHED'
      TOTAL    = NCOL*NROW*NSEC
      DIFF     = MAP_MAX - MAP_MIN
C
      IF(NSEC.LE.1) THEN
C          ****************
C----------*    2D MAP    *
C          ****************
         DO J = 1,NROW
           DO I = 1,NCOL
             N      = ((MAP(I,J,1) - MAP_MIN)/DIFF)*SIZE + 1
             BIN(N) = BIN(N) + 1
           ENDDO
         ENDDO
       ELSE
C          ****************
C----------*    3D MAP    *
C          ****************
         DO K = 1,NSEC
           DO J = 1,NROW
             DO I = 1,NCOL
               N      = ((MAP(I,J,K) - MAP_MIN)/DIFF)*SIZE + 1
               BIN(N) = BIN(N) + 1
             ENDDO
           ENDDO
         ENDDO
      ENDIF
C  ******************************************************************
C  *    COMPUTE FRACTION OF AREA/VOLUME AT EACH LEVEL IN THE MAP    *
C  ******************************************************************
      FRACTION(1) = 100.0*FLOAT(BIN(1))/TOTAL
      DO I = 2,NBINS
        FRACTION(I) = FRACTION(I-1)+100.0*FLOAT(BIN(I))/TOTAL
      ENDDO
      CALL HITCR
      DO I = 1,NBINS
        FRACTION(I) = 100.0 - FRACTION(I)
        CUTOFF      = FLOAT(I-1)*DIFF/FLOAT(NBINS) + MAP_MIN
        WRITE(*,'(I4,I8,F12.1,2F8.2)') I, BIN(I), CUTOFF,
     1 100.0 - FRACTION(I), FRACTION(I)
      ENDDO
      CALL HITCR
C
      RETURN
      END


C  ******************************************************************
C  *                                                                *
C  *               *****  SUBROUTINE MAP_YFLIP  *****               *
C  *                                                                *
C  *    FLIP MAP SECTION ABOUT HORIZONTAL AXIS (Y -> -Y).           *
C  *                                                                *
C  *    FEBRUARY 1993  PURDUE UNIVERSITY VAX 8550, T. BAKER.        *
C  *    LAST MODIFIED 17-DEC-1997 BY T. BAKER (RID OF EM.CMM).      *
C  ******************************************************************
C
      SUBROUTINE MAP_YFLIP (MAP, NCOL, NROW, EM_WRT)
C
      IMPLICIT NONE
C
      INTEGER*4 NCOL, NROW, I, J, JJ, EM_WRT
      REAL*4    MAP(NCOL,NROW), TMP
C ---------------------------------------------------------------------
C
      IF(EM_WRT.EQ.1) THEN
        PRINT *, '***********************************'
        PRINT *, '* Invert MAP about X-axis (Y->-Y) *'
        PRINT *, '***********************************'
      ENDIF
C
      DO I = 1,NCOL
        DO J = 1,(NROW/2)
          JJ        = NROW - (J - 1)
          TMP       = MAP(I,J)
          MAP(I,J)  = MAP(I,JJ)
          MAP(I,JJ) = TMP
        ENDDO        
      ENDDO
C
      RETURN
      END

C  ********************************************************************
C  *                                                                  *
C  *            *****     SUBROUTINE MAP_WRITEASCII     *****         *
C  *                                                                  *
C  *    WRITE ASCII MAP TO DISK FILE.                                 *
C  *                                                                  *
C  *    JUNE 1997 PURDUE UNIVERSITY VAX 3000/400, T. BAKER.           *
C  *    LAST MODIFIED  2-JUN-1997 BY T. BAKER.                        *
C  ********************************************************************
C
      SUBROUTINE MAP_WRITEASCII (LUN_OUT, MAP, NCOL, NROW, NSEC)
C
      IMPLICIT NONE
C
      INTEGER*4 I, II, J, JJ, K, COUNT
      INTEGER*4 NCOL, NROW, NSEC, LUN_OUT, SEC, NPIX_PER_SEC
      REAL*4    MAP(NCOL*NROW,NSEC), TMP(6)
C  ----------------------------------------------------------------------------
C
      NPIX_PER_SEC = NCOL*NROW
C
      DO SEC = 1,NSEC
        WRITE(LUN_OUT,'(I8)') (SEC-1)
        DO J =1,NPIX_PER_SEC,6
          JJ = J - 1
          DO I = 1,6
            K     = JJ + I
            COUNT = I
            IF(K.GT.NPIX_PER_SEC) THEN
               COUNT = COUNT - 1
               WRITE(LUN_OUT,'(6E12.5)') (TMP(II),II=1,COUNT)
               GO TO 200
             ELSE
               TMP(I) = MAP(K,SEC)
            ENDIF
          ENDDO
          WRITE(LUN_OUT,'(6E12.5)') TMP
  200        ENDDO
      ENDDO
C
      RETURN
      END

C  **********************************************************************
C  *                                                                    *
C  *                  *****  SUBROUTINE MAP_1D  *****                   *
C  *                                                                    *
C  *    COMPUTE SPHERICALLY-AVERAGED DENSITY                              *
C  *                                                                    *
C  *    SEPTEMBER 1988  PURDUE UNIVERSITY VAX 8550, T. BAKER.           *
C  *      Converted to Xwindows, November 1993, by Eric Eltzroth            *
C  *      Modified 21 May 1996, by D. Belnap                        *
C  *    MODIFIED 26-SEP-1997 BY T. BAKER: COMMENT OUT DISPLAY OPTION.   *
C  *                                fixed error in output radius value  *
C  **********************************************************************
C
      SUBROUTINE MAP_1D (MAP, NCOL, NROW, NSEC, XCEN, YCEN, ZCEN,
     1                  INTERP, NTERP, MAX_R, RINT, IRNUM, STDEV)
C
      IMPLICIT NONE
C
      include '../include/vax_minmax.inc'
C
      INTEGER*4    NCOL, NROW, NSEC, INUM, IR, INTERP, IPUCK
      INTEGER*4    NTERP, I, J, J1, J2, K, IXTAB, IYTAB, IRNUM(NTERP)
      REAL*4       XCEN, YCEN, ZCEN, RINT(NTERP), RINT_MIN, RINT_MAX
      REAL*4       MAP(NCOL,NROW,NSEC), X, Y, Z, YSQ, ZSQ, MAX_R
      REAL*4       TEST, RSQ_LIM, FRACTION, STDEV(NTERP)
      LOGICAL      OUTFILE, PLOT
      CHARACTER*64 OUT_FILENAME
C -----------------------------------------------------------------------------
C
      WRITE(*,'('' WAIT: Calculation takes a few moments...'')')
C
      INUM    = NTERP
      RSQ_LIM = MAX_R**2
C
      DO I = 1,INUM                      ! INITIALIZE 1D INTERPOLATION ARRAY
        IRNUM(I) = 0
        RINT(I)  = 0.0
        STDEV(I) = 0.0
      ENDDO
C  **************************************************************
C  *    STEP THROUGH IMAGE AND REDUCE TO 1D CIRCULAR AVERAGE    *
C  **************************************************************
      DO K = 1,NSEC
        Z   = FLOAT(K) - ZCEN
        ZSQ = Z*Z
        DO J = 1,NROW
          Y   = FLOAT(J) - YCEN
          YSQ = Y*Y
          DO I = 1,NCOL
             X    = FLOAT(I) - XCEN
             TEST = X*X + YSQ + ZSQ
             IF(TEST.LE.RSQ_LIM) THEN
               IR             = NINT(INTERP*SQRT(TEST)) + 1
               IRNUM(IR)      = IRNUM(IR) + 1
               RINT(IR)       = RINT(IR) + MAP(I,J,K)
             ENDIF
          ENDDO
        ENDDO
      ENDDO
C  ********************************
C  *    NORMALIZE THE 1D ARRAY    *
C  ********************************
      RINT_MIN = R4_MAX
      RINT_MAX = R4_MIN
      DO I = 1,INUM
        IF(IRNUM(I).GT.1) THEN
          RINT(I)     = RINT(I)/FLOAT(IRNUM(I))            ! Get average
        ENDIF
        RINT_MIN = MIN(RINT_MIN,RINT(I))
        RINT_MAX = MAX(RINT_MAX,RINT(I))
      ENDDO
C  ***************************************
C  *    NOW TRY TO FILL IN THE BLANKS    *
C  ***************************************
      DO I = 2,(INUM-2)
        IF(IRNUM(I).EQ.0) THEN
          J1 = I
          DO WHILE(IRNUM(J1).EQ.0 .AND. J1.GT.1)      ! FIND LAST POINT
            J1 = J1 - 1
          ENDDO
          J2 = I
          DO WHILE(IRNUM(J2).EQ.0 .AND. J2.LT.INUM)      ! FIND NEXT POINT
            J2 = J2 + 1
          ENDDO
          FRACTION = FLOAT(J2-I)/FLOAT(J2-J1)
          RINT(I)  = RINT(J1)*FRACTION + RINT(J2)*(1.0-FRACTION)
        ENDIF
      ENDDO
C  ***************************************************************
C  *  Compute standard deviation at each radius where IRNUM > 1  *
C  ***************************************************************
      DO K = 1,NSEC
        Z   = FLOAT(K) - ZCEN
        ZSQ = Z*Z
        DO J = 1,NROW
          Y   = FLOAT(J) - YCEN
          YSQ = Y*Y
          DO I = 1,NCOL
             X    = FLOAT(I) - XCEN
             TEST = X*X + YSQ + ZSQ
             IF(TEST.LE.RSQ_LIM) THEN
               IR        = NINT(INTERP*SQRT(TEST)) + 1
               STDEV(IR) = STDEV(IR) + (RINT(IR)-MAP(I,J,K))**2
             ENDIF
          ENDDO
        ENDDO
      ENDDO
      DO I = 1,INUM
        IF (IRNUM(I).GT.1) THEN
           STDEV(I) = SQRT( STDEV(I) / FLOAT(IRNUM(I)-1) )
         ELSE
           STDEV(I) = 0.0
        ENDIF
      ENDDO
C  ****************************************
C  *  WRITE TO OUTPUT FILE AND TO SCREEN  *
C  ****************************************
      CALL BLANKL (0)
      WRITE(*,'(A,''Write to output file? (<Y>/N): '')') 0
      CALL GET_YESNO (OUTFILE, .TRUE.)
      IF (OUTFILE) THEN
  750        WRITE(*,'(''$Enter output file name:  '')')
        READ  (5,'(A)',ERR=750) OUT_FILENAME
        PRINT *, '**************************************'
        PRINT *, '* Data written to ASCII output file: *'
        PRINT *, '**************************************'
        OPEN (UNIT=8, FILE=OUT_FILENAME, STATUS='NEW',
     1      FORM='FORMATTED', ACCESS='SEQUENTIAL')
      ENDIF
      WRITE(*,800)
      WRITE(*,802) INTERP
      WRITE(*,804)
      IF (OUTFILE) THEN
        WRITE (8,800)
        WRITE (8,802) INTERP
        WRITE (8,804)
      ENDIF
  800      FORMAT ('   Radius    Average   Standard')
  802      FORMAT (' (pixels*',I1,')  Density   Deviation     N')
  804      FORMAT (' -------------------------------------------------')
      DO I = 1,INUM
        IF(ABS(RINT(I)).GT.999999.0 .OR. ABS(RINT(I)).LT.0.0001) THEN
           WRITE(*,815) I-1,RINT(I),STDEV(I),IRNUM(I)
           IF (OUTFILE) WRITE(8,815) I-1,RINT(I),STDEV(I),IRNUM(I)
         ELSE
           WRITE(*,820) I-1,RINT(I),STDEV(I),IRNUM(I)
           IF (OUTFILE) WRITE(8,820) I-1,RINT(I),STDEV(I),IRNUM(I)
        ENDIF
      ENDDO
  815      FORMAT (I6,E14.4,E10.4,I8)
  820      FORMAT (I6,F14.2,F10.2,I8)
      IF (OUTFILE) CLOSE (UNIT=8, STATUS='KEEP')
C  ***************************************
C  *  Option to draw in graphics window  *
C  ***************************************
CCC      PRINT *, 'GRAPHICS NOT AVAILABLE ON THIS MACHINE!'
CCC      WRITE(*,'(''$PLOT ON GRAPHICS SCREEN? Y/<N>: '')')
CCC      CALL GET_YESNO (PLOT, .FALSE.)
CCC      IF (PLOT)
CCC      1  CALL DECW_RADENS_DRAW
CCC      2  (RINT, INUM, RINT_MIN, RINT_MAX, INTERP, 'SAD')
C
      CALL HITCR
      END
C
C  *************************************************************************
C  *                                                                       *
C  *               *****     SUBROUTINE MAP3DT_STORE     *****             *
C  *                                                                       *
C  *    THIS ROUTINE, PREVIOUSLY CALLED MAP3DT_WRITEH, WRITES OUT COMPLEX  *
C  *     INTERMEDIATE RESULTS OF XZ TRANSFORM FROM "ARRAY" TO "ARRAY2".    *
C  *                                                                       *
C  *    FOR SPACE GROUPS HAVING UNIQUE REFLECTIONS WITH H.LT.0 AS WELL AS  *
C  *     H.GE.0 .                                                          *
C  *************************************************************************
C
      SUBROUTINE MAP3DT_STORE (ARRAY,NCOL,NROW,NSEC,HMAX,KMAX,ARRAY2)
C
      IMPLICIT NONE
C
      INTEGER*4 H, K, L, OFFSET, HMAX, KMAX, NCOL, NROW, NSEC, IQ, IP
      COMPLEX   ARRAY(((NCOL+2)/2),NROW,NSEC)
      REAL*4    ARRAY2(2*NSEC*NROW*(HMAX+1))
C  ----------------------------------------------------------------------------
C
C  *************************
C  *    NEGATIVE H ONLY    *
C  *************************
      IQ = NROW - KMAX
  100      IP = IQ + 1
      IQ = IQ + NROW
      IF(IQ.GT.NROW) GO TO 300
      OFFSET = 1
      DO L=1,(HMAX + 1)
        DO K=1,NSEC
          DO H = IP,IQ
            ARRAY2(OFFSET)   = REAL (ARRAY(L,H,K))
            ARRAY2(OFFSET+1) = AIMAG(ARRAY(L,H,K))
            OFFSET = OFFSET + 2
          ENDDO
        ENDDO
      ENDDO
      IF(IQ.LT.NROW) GO TO 100
C  ****************************
C  *    H POSITIVE OR ZERO    *
C  ****************************
      IQ = 0
  200      IP = IQ + 1
      IQ = IQ + NROW
      IF(IQ.GT.KMAX) IQ = KMAX + 1
      OFFSET = 1
      DO L = 1,(HMAX + 1)
        DO K = 1,NSEC
          DO H = IP,IQ
            ARRAY2(OFFSET)   = REAL (ARRAY(L,H,K))
            ARRAY2(OFFSET+1) = AIMAG(ARRAY(L,H,K))
            OFFSET = OFFSET + 2
          ENDDO
        ENDDO
      ENDDO
      IF(IQ.LE.KMAX) GO TO 200
      RETURN
C  **********************
C  *    SPANNING H=0    *
C  **********************
  300      IQ = IQ - NROW
      IF(IQ.GT.KMAX) IQ = KMAX + 1
      OFFSET = 1
      DO L = 1,(HMAX + 1)
        DO K = 1,NSEC
          DO H = IP,NROW
            ARRAY2(OFFSET)   = REAL (ARRAY(L,H,K))
            ARRAY2(OFFSET+1) = AIMAG(ARRAY(L,H,K))
            OFFSET = OFFSET + 2
          ENDDO
          DO H = 1,IQ
            ARRAY2(OFFSET)   = REAL (ARRAY(L,H,K))
            ARRAY2(OFFSET+1) = AIMAG(ARRAY(L,H,K))
            OFFSET = OFFSET + 2
          ENDDO
        ENDDO
      ENDDO
      IF(IQ.LE.KMAX) GO TO 200
C
      RETURN
      END


C  **********************************************************************
C  *                                                                    *
C  *             *****     SUBROUTINE MAP3DT_PASS1     *****            *
C  *                                                                    *
C  *    THIS SUBROUTINE, MODIFIED FROM TEN EYCK'S READ Y, READS IN A    *
C  *     MAP AS Z-SECTIONS IN WHICH X VARIES MOST RAPIDLY AND Y VARIES  *
C  *     MOST SLOWLY.                                                   *
C  *                                                                    *
C  *    LAST MODIFIED 10-AUG-1998 BY TSB (Fill ARRAY and FFT).          *
C  **********************************************************************
C
      SUBROUTINE MAP3DT_PASS1 (MAP3D, ARRAY, NCOL, NROW, NSEC, D, HMAX)
C
      IMPLICIT NONE
C
      INTEGER*4 NCOL, NROW, NSEC, I, J, K, HMAX, D(5), I1, I2, I3
      REAL*4    MAP3D(NCOL*NROW*NSEC), ARRAY((NCOL+2)*NROW*NSEC)
C  ----------------------------------------------------------------------------
C
      WRITE(*,'(A)') ' First Pass'
C
      I1 = 0
      DO K = 1,NSEC
        I2 = (K-1)*NROW*(NCOL+2)
        DO J = 1,NROW
          I3 = I2 + (J-1)*(NCOL+2)
          DO I = 1,NCOL
            I1 = I1 + 1
            I3 = I3 + 1
            ARRAY(I3) = MAP3D(I1)
          ENDDO
        ENDDO
      ENDDO
C
Ca We're overruning this guy in UNIX
Ca      D(1) = (NCOL+2)*NROW*NSEC
Ca  Initially this was the fix - wwhy??????????
Ca      D(1) = (NCOL+1)*NROW*NSEC
      D(1) = (NCOL+2)*NROW*NSEC
      D(2) = 2
      D(3) = D(1)
      D(4) = D(1)
      D(5) = NCOL + 2
      CALL REALFT (ARRAY(1), ARRAY(2), NCOL/2, D)
      D(2) = NCOL + 2
      D(3) = NROW*(NCOL+2)
      D(4) = 2*(HMAX+1)
      D(5) = 2
      CALL CMPLFT (ARRAY(1), ARRAY(2), NROW, D)
C
      RETURN
      END

C  **********************************************************************
C  *                                                                    *
C  *             *****     SUBROUTINE MAP3DT_PASS2     *****            *
C  *                                                                    *
C  *    THIS SUBROUTINE, MODIFIED FROM TEN EYCK'S READ_H, READS BACK    *
C  *     BACK THE COMPLEX INTERMEDIATE RESULTS OF XZ TRANSFORM AS H -   *
C  *     PLANES, WHEN THERE HAS BEEN NO TRANSPOSITION AND INTERLEAVING  *
C  *     IT ALLOWS NO SCRATCH LOCATION ON Y                             *
C  *                                                                    *
C  *    LAST MODIFIED 11-AUG-1998 BY TSB (Fill ARRAY and FFT).          *
C  **********************************************************************
C
      SUBROUTINE MAP3DT_PASS2 (ARRAY, ARRAY2, NSEC, HMAX, KMAX, D)
C
      IMPLICIT NONE
C
      INTEGER*4 NSEC, H, K, L, OFFSET, HMAX, KMAX, D(5), I1, I2, I3
      REAL*4    ARRAY (2*NSEC*(2*KMAX+1)*(HMAX+1))
      REAL*4    ARRAY2(2*NSEC*(2*KMAX+1)*(HMAX+1))
C  ----------------------------------------------------------------------------
C
      WRITE(*,'(A)') '+Second Pass'
C
      OFFSET = 1
      DO H = 1,(HMAX+1)
        I1 = (H-1)*2*NSEC
        DO L = 1,NSEC
          I2 = I1 + (L-1)*2 + 1
          DO K = 1,(2*KMAX+1)
            I3 = I2 + (K-1)*(HMAX+1)*2*NSEC
            ARRAY(I3)   = ARRAY2(OFFSET)
            ARRAY(I3+1) = ARRAY2(OFFSET+1)
            OFFSET = OFFSET + 2
          ENDDO
        ENDDO
      ENDDO
C
      D(1)  = 2*NSEC*(HMAX+1)*(2*KMAX+1)
      D(2)  = 2
      D(3)  = D(1)
      D(4)  = D(1)
      D(5)  = 2*NSEC
      CALL CMPLFT (ARRAY(1), ARRAY(2), NSEC, D)
C
      RETURN
      END
